import doctest

from ofxstatement.plugins.litas_esis import LitasEsisCsvStatementParser


def doctest_LitasEsisCsvStatementParser():
    """Test LitasEsisCsvStatementParser

    Open sample csv to parse
        >>> import os
        >>> csvfile = os.path.join(os.path.dirname(__file__),
        ...                        'samples', 'litas_esis.acc')

    Create parser object and parse:
        >>> fin = open(csvfile, 'r', encoding='cp1257')
        >>> parser = LitasEsisCsvStatementParser(fin)
        >>> statement = parser.parse()

    Check what we've got:
        >>> statement.account_id
        'LT000000000000000000LTL'
        >>> len(statement.lines)
        7
        >>> statement.start_balance
        251.75
        >>> statement.start_date
        datetime.datetime(2012, 1, 1, 0, 0)
        >>> statement.end_balance
        74.83
        >>> statement.end_date
        datetime.datetime(2012, 3, 4, 0, 0)
        >>> statement.currency
        'LTL'

    Check first line:
        >>> l = statement.lines[0]
        >>> l.amount
        -1.0
        >>> l.payee
        'AB DNB BANKAS'
        >>> l.memo
        'Mokestis už sąskaitos aptarnavimą'
        >>> l.id
        '1987555498'

    Check credit line:
        >>> l = statement.lines[3]
        >>> l.amount
        300.0
        >>> l.payee
        'LINUS TORVALDS'
        >>> l.memo
        'Hello World'
        >>> l.id
        '2003969289'

    """
def doctest_LitasEsisCsvStatementParser_swap_payee_memo():
    """Test ability to swap payee and memo in LitasEsisCsvStatementParser

    Open sample csv to parse
        >>> import os
        >>> csvfile = os.path.join(os.path.dirname(__file__),
        ...                        'samples', 'litas_esis.acc')

    Create parser object and parse:
        >>> fin = open(csvfile, 'r', encoding='cp1257')
        >>> parser = LitasEsisCsvStatementParser(fin)
        >>> parser.swap_payee_and_memo()
        >>> statement = parser.parse()

    Memo and payee should be swapped
        >>> l = statement.lines[3]
        >>> l.memo
        'LINUS TORVALDS'
        >>> l.payee
        'Hello World'
    """

def test_suite(*args):
    return doctest.DocTestSuite(optionflags=(doctest.NORMALIZE_WHITESPACE|
                                             doctest.ELLIPSIS|
                                             doctest.REPORT_ONLY_FIRST_FAILURE|
                                             doctest.REPORT_NDIFF
                                             ))
load_tests = test_suite



# -*- coding: utf-8 -*-

import re
import itertools
import logging
import locale

from datetime import datetime
from openpyxl import load_workbook

from contextlib import contextmanager
from ofxstatement.parser import StatementParser
from ofxstatement.plugin import Plugin
from ofxstatement.statement import Statement, StatementLine, generate_transaction_id


def take(n, iterable):
    """Return first n items of the iterable as a list."""
    return list(itertools.islice(iterable, n))


@contextmanager
def scoped_setlocale(category, loc=None):
    """Scoped version of locale.setlocale()"""
    orig = locale.getlocale(category)
    try:
        yield locale.setlocale(category, loc)
    finally:
        locale.setlocale(category, orig)


def atof(string, loc=None):
    """Locale aware atof function for our parser."""
    with scoped_setlocale(locale.LC_NUMERIC, loc):
        return locale.atof(string)


class SebStatementParser(StatementParser):
    date_format = '%Y-%m-%d'
    bank_id = 'SEB'
    currency_id = 'SEK'
    footer_regexps = [
        '^Datum:  -',
        '^Datum: ([0-9]{4}-[0-9]{2}-[0-9]{2}) - ([0-9]{4}-[0-9]{2}-[0-9]{2})$'
    ]

    def __init__(self, fin, locale=None, brief=False):
        """
        Create a new SebStatementParser instance.

        :param fin: filename to create parser for
        :param brief: whenever to attempt replace description with a brief version i.e. all extra info removed
        """

        self.locale = locale
        self.brief = brief

        self.workbook = load_workbook(filename=fin, read_only=True)
        self.validate()

        self.statement = self.parse_statement()

    def validate(self):
        """
        Naive validation to make sure that xlsx document is structured the way it was
        when this parser was written.

        :raises ValueError if workbook has invalid format
        """

        try:
            self._validate()
        except AssertionError as e:
            raise ValueError(e)

    def _validate(self):
        sheet = self.workbook.active

        logging.info('Checking that sheet has at least 5 rows.')
        rows = take(5, sheet.iter_rows())
        assert len(rows) == 5

        logging.info('Verifying that every row has 6 cells.')
        assert type(rows) == list
        for row in rows:
            assert len(row) == 6

        logging.info('Extracting values from every cell.')
        rows = [[c.value for c in row] for row in rows]

        logging.info('Verifying summary header.')
        summary_header_row = rows[0]
        assert ['Saldo', 'Disponibelt belopp', 'Beviljad kredit', None, None] == summary_header_row[1:]

        logging.info('Verifying account id.')
        summary_account_row = rows[1]
        account_id = summary_account_row[0]
        assert re.match('^[0-9]+$', account_id)
        assert [None, None] == summary_account_row[-2:]

        def is_footer(row):
            for r in self.footer_regexps:
                if re.match(r, row[0]):
                    return True
            return False

        logging.info('Verifying summary footer.')
        summary_footer_row = rows[2]
        assert is_footer(summary_footer_row)
        assert [None, None, None, None, None] == summary_footer_row[1:]

        logging.info('Verifying empty row.')
        empty_row = rows[3]
        assert [None, None, None, None, None, None] == empty_row

        logging.info('Verifying statements header.')
        statement_header_row = rows[4]
        assert ['Bokföringsdatum', 'Valutadatum', 'Verifikationsnummer', 'Text/mottagare', 'Belopp', 'Saldo'] == statement_header_row

        logging.info('Everything is OK!')

    def parse_statement(self):
        statement = Statement()
        sheet = self.workbook.active

        # We need only first 3 rows here.
        rows = take(3, sheet.iter_rows())
        rows = [[c.value for c in row] for row in rows]

        assert len(rows) == 3
        header_row, account_row, footer_row = rows

        account_id, saldo, disponibelt_belopp, beviljad_kredit, _1, _2 = account_row
        statement.account_id = account_id
        statement.end_balance = atof(saldo, self.locale)
        statement.bank_id = self.bank_id
        statement.currency = self.currency_id

        for r in self.footer_regexps:
            m = re.match(r, footer_row[0])
            if m and m.groups():
                part_from, part_to = m.groups()
                statement.start_date = self.parse_datetime(part_from)
                statement.end_date = self.parse_datetime(part_to)

        return statement

    def split_records(self):
        sheet = self.workbook.active

        # Skip first 5 rows. Headers they are.
        for row in itertools.islice(sheet.iter_rows(), 5, None):
            yield [c.value for c in row]

    def parse_record(self, row):
        row = take(5, row)

        stmt_line = StatementLine()
        stmt_line.date = self.parse_datetime(row[0])
        _ = self.parse_datetime(row[1])  # TODO: ???
        stmt_line.refnum = row[2]
        stmt_line.memo = row[3]
        stmt_line.amount = row[4]

        #
        # Looks like SEB formats description for card transactions so it includes the actual purchase date
        # within e.g. 'WIRSTRÖMS PU/14-12-31' and it means that description is 'WIRSTRÖMS PU' while the actual
        # card operation is 2014-12-31.
        #
        # P.S. Wirströms Irish Pub is our favorite pub in Stockholm: http://www.wirstromspub.se
        #
        m = re.match('(.*)/([0-9]{2}-[0-9]{2}-[0-9]{2})$', stmt_line.memo)
        if m:
            card_memo, card_date = m.groups()
            if self.brief:
                stmt_line.memo = card_memo
            stmt_line.date_user = datetime.strptime(card_date, '%y-%m-%d')

        stmt_line.id = generate_transaction_id(stmt_line)
        return stmt_line


def parse_bool(value):
    if value in ('True', 'true', '1'):
        return True
    if value in ('False', 'false', '0'):
        return False
    raise ValueError("Can't parse boolean value: %s" % value)


class SebPlugin(Plugin):
    def get_parser(self, fin):
        kwargs = {
            'locale': 'sv_SE'
        }
        if self.settings:
            if 'locale' in self.settings:
                kwargs['locale'] = parse_bool(self.settings.get('locale'))
            if 'brief' in self.settings:
                kwargs['brief'] = parse_bool(self.settings.get('brief'))
        return SebStatementParser(fin, **kwargs)



import xlrd
from ofxstatement.parser import StatementParser
from ofxstatement.plugin import Plugin
from ofxstatement.statement import Statement, StatementLine, generate_transaction_id


class LansforsakringarPlugin(Plugin):
    """Länsförsäkringar <https://www.lansforsakringar.se>"""

    def get_parser(self, filename):
        bank_id = self.settings.get('bank', 'ELLFSESS')
        account_id = self.settings.get('account')
        return LansforsakringarParser(filename, bank_id, account_id)


class LansforsakringarParser(StatementParser):
    statement = Statement(currency='SEK')

    def __init__(self, filename, bank_id, account_id):
        self.filename = filename
        self.statement.bank_id = bank_id
        self.statement.account_id = account_id
        self.sheet = None
        self.row_num = 0

    def parse(self):
        with xlrd.open_workbook(self.filename) as book:
            self.sheet = book.sheet_by_index(0)
            return super().parse()

    def split_records(self):
        rows = self.sheet.get_rows()
        next(rows)  # statement date
        next(rows)  # headers
        return rows

    def parse_record(self, row):
        self.row_num += 1
        line = StatementLine()
        line.date = self.parse_datetime(row[0].value)
        line.date_user = self.parse_datetime(row[1].value)
        line.refnum = str(self.row_num)
        line.memo = row[2].value
        line.amount = row[3].value
        line.trntype = self.get_type(line)
        line.id = generate_transaction_id(line)
        return line

    @staticmethod
    def get_type(line):
        if line.amount > 0:
            return 'CREDIT'
        elif line.amount < 0:
            return 'DEBIT'
        else:
            return 'OTHER'



#!/usr/bin/python3
"""Setup
"""
try:
    from setuptools import setup
except ImportError:
    from distutils.core import setup

with open('README.rst') as readme_file:
    readme = readme_file.read()

with open('HISTORY.rst') as history_file:
    history = history_file.read().replace('.. :changelog:', '')

requirements = [
    'ofxstatement'
]

setup(name='ofxstatement-simple',
      version='0.1.0',
      author="Chris Mayes",
      author_email="cmayes@cmay.es",
      url="https://github.com/cmayes/ofxstatement-simple",
      description="Simple (the bank) plugin for ofxstatement",
      long_description=readme + '\n\n' + history,
      license="GPLv3",
      keywords=["ofx", "banking", "statement"],
      classifiers=[
          'Development Status :: 3 - Alpha',
          'Programming Language :: Python :: 3',
          'Natural Language :: English',
          'Topic :: Office/Business :: Financial :: Accounting',
          'Topic :: Utilities',
          'Environment :: Console',
          'Operating System :: OS Independent',
          'License :: OSI Approved :: GNU Affero General Public License v3'],
      packages=[
          'ofxstatement', 'ofxstatement.plugins',
      ],
      package_dir={'ofxstatement': 'ofxstatement',
                   'ofxstatement.plugins': 'ofxstatement/plugins'},
      namespace_packages=["ofxstatement", "ofxstatement.plugins"],
      entry_points={
          'ofxstatement':
              ['simple = ofxstatement.plugins.simple:SimpleBankPlugin']
      },
      install_requires=requirements,
      include_package_data=True,
      zip_safe=True,
      test_suite='tests',
      tests_require=requirements
      )



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

import os
import time
import click
import logging

from ofxstatement.ofx import OfxWriter
from ofxstatement.plugins import argenta

@click.command()
@click.argument('path', type=click.Path(exists=True, readable=True))
@click.option('--debug', is_flag=True, default=False, 
    help='Log assertions and statement lines. No file is written.')
def convert(path, debug):
    """Parse and write transactions from Argenta_iban_date.xlsx file. The given file is renamed."""

    level = logging.DEBUG if debug else logging.INFO
    logging.basicConfig(level=level, format='[%(levelname)s] %(message)s')

    with argenta.ArgentaPlugin(ui=None, settings=None).get_parser(path) as parser:
        statement = parser.parse()
        logging.debug('Statement has been parsed.')

    if debug:
        logging.debug('Statement.lines:')
        for line in statement.lines:
            print(line)
        return
    
    date_suffix = statement.lines[-1].date.strftime("-%y%m%d")
    ext = os.path.splitext(path)[1]
    dir = os.path.dirname(path)
    base_filename = os.path.join(dir, statement.account_id + date_suffix)
    output_file = base_filename + '.ofx'

    with open(output_file, 'w') as out:
        writer = OfxWriter(statement)
        out.write(writer.toxml())
        logging.info('Statement has been written to ' + output_file)
        
    path_new = base_filename + ext
    os.rename(path, path_new)
    logging.info('Original file has been renamed to ' + path_new)

if __name__ == '__main__':
    convert()



import csv
import re

from ofxstatement import statement
from ofxstatement.parser import CsvStatementParser
from ofxstatement.plugin import Plugin
from ofxstatement.statement import Statement, StatementLine, recalculate_balance


class MillenniumPlugin(Plugin):
    """Millennium (www.bankmillennium.pl) bank CSV history plugin
    """

    def get_parser(self, filename):
        encoding = self.settings.get('charset', 'utf8')
        f = open(filename, "r", encoding=encoding)
        parser = MillenniumParser(f)
        parser.statement.currency = self.settings.get('currency', 'PLN')
        parser.statement.account_id = self.settings.get('account', '')
        parser.statement.bank_id = self.settings.get('bank', 'BIGBPLPW')
        return parser

class MillenniumParser(CsvStatementParser):
    mappings = {"date": 1}
    
    date_format = "%Y-%m-%d"
    
    def parse(self):
        stmt = super(MillenniumParser, self).parse()
        recalculate_balance(stmt)
        return stmt
    
    def split_records(self):
        return csv.reader(self.fin, delimiter=',', quotechar='"')
    
    def parse_record(self, line):
        
        if self.cur_record < 2:
            return
        acc = re.sub("\s", "", line[0]) 
        if not self.statement.account_id:
            self.statement.account_id = acc
        elif acc != self.statement.account_id:
            return
        
        sl = super(MillenniumParser, self).parse_record(line)
        
        sl_type = line[3]
        sl_acc_no = re.sub("\s|[a-zA-Z]", "", line[4])
        sl_name = line[5]
        sl_desc = line[6]
        sl_charge = line[7]
        sl_credit = line[8]
        #sl_saldo = line[9]
        
        if sl_charge:
            sl.amount = self.parse_float(sl_charge)
        elif sl_credit:
            sl.amount = self.parse_float(sl_credit)
            
        if not sl.amount:
            raise ValueError("Could not parse transaction amount")
        
        sl.memo = re.sub("\s+", " ", ' '.join([sl_acc_no, sl_name]).strip())
        sl.payee = re.sub("\s+", " ", ' - '.join([sl_type, sl_desc]).strip())
        
        if sl_type.startswith("PRZELEW"):
            sl.trntype = "XFER"
        elif sl_type.startswith("WYPŁATA"):
            sl.trntype = "ATM"
        elif sl_type.startswith("TRANSAKCJA KARTĄ"):
            sl.trntype = "DEBIT"
        elif sl_type == "OBCIĄŻENIE":
            if sl_desc.startswith("PODATEK"):
                sl.trntype = "FEE"
            elif sl_desc.startswith("OPŁATA"):
                sl.trntype = "SRVCHG"
            else:
                sl.trntype = "DEBIT"
        elif sl_type == "UZNANIE":
            if sl_desc.startswith("KAPITALIZACJA"):
                sl.trntype = "INT"
            else:
                sl.trntype = "CREDIT"
        
        sl.id = statement.generate_transaction_id(sl)
        
        return sl
    


import csv

from ofxstatement import statement
from ofxstatement.parser import CsvStatementParser
from ofxstatement.plugin import Plugin
from ofxstatement.parser import StatementParser
from ofxstatement.statement import StatementLine


class KeytradePlugin(Plugin):
    """Sample plugin (for developers only)
    """

    def get_parser(self, filename):
        f = open(filename, 'r', encoding=self.settings.get("charset", "ISO-8859-1"))
        parser = KeytradeParser(f)
        parser.statement.bank_id = "KeytradeBank"
        return parser


class KeytradeParser(CsvStatementParser):

    date_format = "%d.%m.%Y"

    mappings = {
        'check_no': 0,
        'date': 1,
        'payee': 3,
        'memo': 4,
        'amount': 5
    }

    def parse(self):
        """Main entry point for parsers

        super() implementation will call to split_records and parse_record to
        process the file.
        """
        stmt = super(KeytradeParser, self).parse()
        statement.recalculate_balance(stmt)
        return stmt

    def split_records(self):
        """Return iterable object consisting of a line per transaction
        """
        reader = csv.reader(self.fin, delimiter=";")
        next(reader, None)
        return reader

    def fix_amount(self, amount):
        return amount.replace('.', '').replace(',', '.')

    def parse_record(self, line):
        """Parse given transaction line and return StatementLine object
        """
        transaction_id = line[0]
        date = line[1]
        date_value = line[2]
        account_to = line[3]
        description = line[4]
        line[5] = self.fix_amount(line[5])
        currency = line[6]

        stmtline = super(KeytradeParser, self).parse_record(line)
        stmtline.trntype = 'DEBIT' if stmtline.amount < 0 else 'CREDIT'

        return stmtline



__author__ = 'Chris Mayes'
__email__ = 'cmayes@cmay.es'
__version__ = '0.1.0'

from datetime import datetime
import json
import logging

from ofxstatement.plugin import Plugin
from ofxstatement.parser import StatementParser
from ofxstatement.statement import StatementLine, Statement

logging.basicConfig(level=logging.DEBUG)
logger = logging.getLogger('simple')


class SimpleBankPlugin(Plugin):
    """Plugin for Simple (the bank) JSON statements.

    Note that you can specify 'bank' and 'account' in ofxstatement's configuration file (accessible
    using the `ofxstatement edit-config` command or directly at
    `~/.local/share/ofxstatement/config.ini` (on Linux, at least).  Setting these values under the
    "simple section" (i.e. "[simple]") makes it easier for your personal finance application
    to recognize which account the file's data belongs to.

    Also note that transactions for zero amounts are filtered by default.  If you wish to include
    zero-amount transactions, set 'zero_filter' to 'false' in your settings.
    """

    def get_parser(self, fin):
        parser = SimpleBankJsonParser(fin, self.settings.get('charset', 'utf-8'))
        parser.statement.bank_id = self.settings.get('bank', 'Simple')
        parser.statement.account_id = self.settings.get('account', None)
        parser.statement.filter_zeros = str2bool(self.settings.get('filter_zeros', "true"))
        return parser


def process_balances(stmt):
    """
    Based on `ofxstatement.statement.recalculate_balance`. Uses the "running_balance" field
    from the Simple transaction data for fetching the start and end balance for this statement.
    If the statement has no transactions, it is returned unmodified.

    :param stmt: The statement data to process.
    :return: The statement with modified start_balance, end_balance, start_date, and_end_date.
    """
    if not stmt.lines:
        return False
    stmt.lines.sort(key=lambda x: x.date)
    first_line = stmt.lines[0]
    stmt.start_balance = (first_line.end_balance - first_line.amount)
    stmt.end_balance = stmt.lines[-1].end_balance
    stmt.start_date = min(sl.date for sl in stmt.lines)
    stmt.end_date = max(sl.date for sl in stmt.lines)
    return True




class SimpleBankJsonParser(StatementParser):

    def split_records(self):
        """Nothing needed for JSON format."""
        pass

    def __init__(self, fin, encoding):
        self.encoding = encoding
        self.fin = fin
        self.statement = Statement()

    def parse(self):
        """Read and parse statement
            Return Statement object

            May raise exceptions.ParseException on malformed input.
            """
        with open(self.fin, 'r', encoding=self.encoding) as fhandle:
            data = json.load(fhandle)
            for line in data['transactions']:
                stmt_line = self.parse_record(line)
                if stmt_line:
                    stmt_line.assert_valid()
                    self.statement.lines.append(stmt_line)
            process_balances(self.statement)
            return self.statement

    def parse_record(self, line):
        """Extracts the transaction data from the given line parsed from the JSON file.

        :param line: A transaction line from the parsed JSON file.
        :return: A new StatementLine filled by the given line's data.
        """
        amount = to_float(line['amounts']['amount'])
        if self.statement.filter_zeros and is_zero(amount):
            return None

        memo = line['description']
        datetime = ts_to_datetime(line['times']['when_recorded'])
        id = line['uuid']
        ledger_amount = convert_debit(amount, line['bookkeeping_type'])
        stmt_line = StatementLine(id=id, date=datetime, memo=memo, amount=ledger_amount)
        stmt_line.end_balance = to_float(line['running_balance'])
        return stmt_line


def is_zero(fval):
    """Returns whether the given float is an approximation of zero."""
    return abs(fval - 0.0) <= 0.000001


def str2bool(v):
    """Converts a string to a boolean value.  From http://stackoverflow.com/a/715468"""
    return v.lower() in ("yes", "true", "t", "1")


def to_float(json_int):
    """Converts the given integer to a float by dividing it by 10,000 and creating a
    float from the result."""
    return float(json_int/10000)


def ts_to_datetime(epoch):
    """Converts the given epoch time in milliseconds to a datetime instance."""
    return datetime.fromtimestamp(epoch/1000)


def convert_debit(amount, book_type):
    """Converts the amount to negative if the book_type is 'debit'."""
    if book_type is not None and 'debit' == book_type.lower():
        return -amount
    return amount



import csv
import types
from io import StringIO

from ofxstatement.parser import CsvStatementParser
from ofxstatement.statement import StatementLine, BankAccount
from . import TRANSACTION_TYPES

# The bank for some reason has varied the column labeling a lot.
# By collecting first lines of known CSV exports as signatures
# we catch any future changes to the CSV export format.

SIGNATURES = (
'Kirjauspäivä;Arvopäivä;Määrä  MF NR;"Laji";Selitys;\
Saaja/Maksaja;Saajan tilinumero ja pankin BIC;Viite;Viesti;Arkistointitunnus;',
'Kirjauspäivä;Arvopäivä;Määrä  NDEAF;"Laji";Selitys;\
Saaja/Maksaja;Saajan tilinumero ja pankin BIC;Viite;Viesti;Arkistointitunnus;',
"Kirjauspäivä;Arvopäivä;Määrä  EUROA;\"Laji\";Selitys;\
Saaja/Maksaja;Saajan tilinumero ja pankin BIC;Viite;Viesti;Arkistointitunnus;",
"Kirjauspäivä;Arvopäivä;Määrä  ;\"Laji\";Selitys;Saaja/Maksaja;\
Saajan tilinumero ja pankin BIC;Viite;Viesti;Arkistointitunnus;",
"Kirjauspäivä;Arvopäivä;Määrä EUROA;Tapahtumalajikoodi;Selitys;\
Saaja/Maksaja;Saajan tilinumero;Viite;Viesti;Arkistotunnus;",
"Kirjauspäivä;Arvopäivä;Määrä EUROA;Laji;Selitys;Saaja/Maksaja;\
Saajan tilinumero ja pankin BIC;Viite;Viesti;Arkistointitunnus;",
"Kirjauspäivä;Arvopäivä;Määrä EUROA;Laji;Selitys;Saaja/Maksaja;\
Saajan tilinumero ja pankin BIC;Viite;Viesti;Arkistointitunnus",
)


class CustomStatementLine(StatementLine):
    "don't print the check number"

    def __str__(self):
        return """
        ID: %s, date: %s, amount: %s, payee: %s
        memo: %s
        """ % (self.id, self.date, self.amount, self.payee, self.memo)


class OPCsvStatementParser(CsvStatementParser):
    "parser for various variations with common field semantics"
    
    mappings = {
       "date":1, "amount":2, "trntype":4, "payee":5,
       "bank_account_to":6, "refnum":7, "memo":8, "id":9
    }

    date_format = "%d.%m.%Y"

    def __init__(self, fin):
        sin=StringIO()
        for l in fin:
           # Some versions from 2011 have broken CSV...
           sin.write(l.replace("&amp;amp;", "&"))
        sin.seek(0)
        super().__init__(sin)

    def split_records(self):
        return csv.reader(self.fin, delimiter=';', quotechar='"')

    def parse_value(self, value, field):
       if field == "bank_account_to":
          return BankAccount("", value)
       else:
          return super().parse_value(value, field)

    def parse_record(self, line):
        #Free Headerline
        if self.cur_record <= 1:
            return None

        # Change decimalsign from , to .
        line[2] = line[2].replace(',', '.')

        # Set transaction type
        line[4] = TRANSACTION_TYPES[line[4]]

        stmt_line = CustomStatementLine()
        for field, col in self.mappings.items():
            if col >= len(line):
                raise ValueError("Cannot find column %s in line of %s items "
                                 % (col, len(line)))
            rawvalue = line[col]
            value = self.parse_value(rawvalue, field)
            setattr(stmt_line, field, value)
        return stmt_line



# Transaction type mapping for Osuuspankki statement parsers

TRANSACTION_TYPES = {
   "TILISIIRTO": "XFER",
   "PALVELUMAKSU": "FEE",
   "PKORTTIMAKSU": "POS",
   "VIITESIIRTO": "XFER",
   "PIKASIIRTO": "XFER",
   "ETUUS": "XFER",
   "LAPSILISÄ": "XFER",
   "KORVAUS": "XFER",
   "AUTOMAATTINOSTO": "ATM",
   "MAKSUPALVELU": "REPEATPMT",
   "LUOTON MAKSU": "REPEATPMT",
   "TALLETUSKORKO": "INT",
   "KORKO": "INT",
   "LÄHDEVERO": "FEE",
   "SUORAVELOITUS": "DIRECTDEBIT",
   "PANO": "DEP",
   "PERUUTUS": "DIRECTDEP",
   "VUOKRA": "PAYMENT",
   "VERONPALAUTUS": "DIRECTDEP",
   "PALKKA": "DIRECTDEP",
   "ARVOPAPERI": "OTHER",
   "TALLETUSAUTOM.": "XFER"
}

#   Other types available: 'CREDIT', 'DEBIT', 'INT', 'DIV', 'FEE', 'DEP',
#   'ATM', 'CHECK', 'CASH', 'DIRECTDEP', 'DIRECTDEBIT', 'REPEATPMT', 'OTHER'





#!/usr/bin/python3
from setuptools import find_packages
from distutils.core import setup

version = "1.0.3"

with open('README.rst') as f:
    long_description = f.read()

setup(name='ofxstatement-lithuanian',
      version=version,
      author="Andrey Lebedev",
      author_email="andrey@lebedev.lt",
      url="https://github.com/kedder/ofxstatement-lithuanian",
      description=("Statement parsers for banks, operatiing in Lithuania"),
      long_description=long_description,
      license="GPLv3",
      keywords=["ofxstatement", "lithuanian"],
      classifiers=[
          'Development Status :: 3 - Alpha',
          'Programming Language :: Python :: 3',
          'Natural Language :: English',
          'Topic :: Office/Business :: Financial :: Accounting',
          'Topic :: Utilities',
          'Environment :: Console',
          'Operating System :: OS Independent',
          'License :: OSI Approved :: GNU Affero General Public License v3'],
      packages=find_packages('src'),
      package_dir={'': 'src'},
      namespace_packages=["ofxstatement", "ofxstatement.plugins"],
      entry_points={
          'ofxstatement':
          ['litas-esis = ofxstatement.plugins.litas_esis:LitasEsisPlugin',
           'swedbank = ofxstatement.plugins.swedbank:SwedbankPlugin',
           'danske = ofxstatement.plugins.danske:DanskePlugin']
          },
      install_requires=['ofxstatement'],
      test_suite="ofxstatement.plugins.tests",
      include_package_data=True,
      zip_safe=True
      )



import csv

from ofxstatement.plugin import Plugin
from ofxstatement.parser import CsvStatementParser
from ofxstatement.statement import StatementLine


class FrankfurterSparkasse1822Plugin(Plugin):
    def get_parser(self, filename):
        encoding = self.settings.get('charset', 'iso-8859-1')
        with open(filename, 'r', encoding=encoding) as f:
            lines = f.readlines()
        parser = FrankfurterSparkasse1822Parser(lines)
        parser.statement.account_id = self.settings['account']
        parser.statement.bank_id = self.settings.get('bank', '50050201')
        parser.statement.currency = self.settings.get('currency', 'EUR')
        return parser


class FrankfurterSparkasse1822Parser(CsvStatementParser):
    """
    This plugin tries to parse the provided CSV data into the same
    format, as the discontinued OFX export
    """
    date_format = "%d.%m.%Y"

    def split_records(self):
        return csv.reader(self.fin, delimiter=';')

    def parse_float(self, f):
        """ convert a number in german localization (e.g. 1.234,56) to float """
        return float(f.replace('.', '').replace(',', '.'))

    def parse_record(self, line):
        # FIXME: add header validation
        if self.cur_record < 2:
            return None
        if len(line) < 3:
            """e.g.: ['# 1 vorgemerkte Umsätze nicht angezeigt']"""
            return None
        if not line[2]:
            return None
        sl = StatementLine()
        sl.id = line[1]
        sl.date = self.parse_datetime(line[2])
        sl.amount = self.parse_float(line[4])
        sl.trntype = 'DEBIT' if sl.amount < 0 else 'CREDIT'
        sl.payee = line[7]
        # check for special transactions
        if line[6] == "Entgeltabschluss":
            sl.memo = "%s: %s %s" % (line[6], line[13], line[14])
        elif line[6] == "Wertpapiere" or line[7] == "KREDITKARTENABRECHNUNG":
            sl.memo = "(%s/%s): %s" % (line[8], line[9], " ".join(line[15:]).strip())
        elif not line[8] and not line[9]:
            # empty transaction
            print("empty", line)
            return None
        else:
            sl.memo = "(%s/%s): %s" % (line[8], line[9], " ".join(e for e in line[13:] if e).strip())

        return sl




import csv

from ofxstatement.parser import CsvStatementParser
from ofxstatement.plugin import Plugin
from ofxstatement.statement import generate_transaction_id


class ALBankPlugin(Plugin):
    """Arbejdernes Landsbank <https://www.al-bank.dk>"""

    def get_parser(self, filename):
        bank_id = self.settings.get('bank', 'ALBADKKK')
        account_id = self.settings.get('account')
        return ALBankParser(open(filename, mode='r', encoding='ISO-8859-15'), bank_id, account_id)


class ALBankParser(CsvStatementParser):
    date_format = '%d-%m-%Y'

    def __init__(self, fin, bank_id, account_id):
        super().__init__(fin)
        self.statement.bank_id = bank_id
        self.statement.account_id = account_id
        self.statement.currency = 'DKK'
        self.headers = {}
        self.row_num = 0

    def split_records(self):
        reader = csv.reader(self.fin, delimiter=';')

        headers = next(reader)
        for i, header in enumerate(headers):
            self.headers[header] = i
            if header == 'Dato':
                self.mappings['date'] = i
            elif header == 'Valørdato':
                self.mappings['date_user'] = i
            elif header == 'Tekst':
                self.mappings['memo'] = i
            elif header == 'Beløb':
                self.mappings['amount'] = i

        return reader

    def parse_record(self, line):
        r = super().parse_record(line)
        self.row_num += 1

        r.refnum = str(self.row_num)
        r.trntype = self.get_type(r)
        if r.date_user:
            r.date_user = self.parse_datetime(r.date_user)
        r.memo = self.concat(r.memo, line, 'Tekst til modtager')
        r.memo = self.concat(r.memo, line, 'Supp. tekst til modtager')
        if r.amount < 0:
            r.memo = self.concat(r.memo, line, 'Modtagernavn')
            r.memo = self.concat(r.memo, line, 'Modtagerkonto')
        elif r.amount > 0:
            r.memo = self.concat(r.memo, line, 'Indbetaler')
        r.id = generate_transaction_id(r)

        return r

    def parse_float(self, value):
        value = value.replace(',', '.')
        return super().parse_float(value)

    def concat(self, text, line, header):
        result = text or ''
        value = self.get_value(line, header)
        if value:
            components = result.split(' | ')
            if value not in components:
                if result:
                    result += ' | '
                result += value
        return result

    def get_value(self, line, header):
        if header in self.headers:
            index = self.headers[header]
            value = line[index]
            if value:
                return value.strip()
        return ''

    @staticmethod
    def get_type(line):
        if line.amount > 0:
            return 'CREDIT'
        elif line.amount < 0:
            return 'DEBIT'
        else:
            return 'OTHER'



import os
import datetime

import pytest

from ofxstatement import exceptions
from ofxstatement.ui import UI
from ofxstatement.plugins.iso20022 import Iso20022Plugin


HERE = os.path.dirname(__file__)
SAMPLES_DIR = os.path.join(HERE, 'samples')


def test_parse_simple():
    # GIVEN
    plugin = Iso20022Plugin(UI(), {})

    parser = plugin.get_parser(os.path.join(SAMPLES_DIR, 'simple.xml'))

    # WHEN
    stmt = parser.parse()

    # THEN
    assert stmt is not None

    assert stmt.account_id == 'LT000000000000000000'
    assert stmt.currency == 'EUR'
    assert stmt.bank_id == 'AGBLLT2XXXX'
    assert stmt.end_balance == 125.52
    assert stmt.end_date == datetime.datetime(2015, 12, 31, 0, 0)
    assert stmt.start_balance == 306.53
    assert stmt.start_date == datetime.datetime(2015, 12, 1, 0, 0)

    assert len(stmt.lines) == 4

    assert all(l.amount for l in stmt.lines)

    line0 = stmt.lines[0]

    assert line0.amount == -0.29
    assert line0.memo == u'Sąskaitos aptarnavimo mokestis'
    assert line0.date == datetime.datetime(2016, 1, 1, 0, 0)
    assert line0.date_user == datetime.datetime(2015, 12, 31, 0, 0)
    assert line0.payee == u'AB DNB Bankas'
    assert line0.refnum == 'FC1261858984'


def test_parse_unconfigured_currency():
    # GIVEN
    plugin = Iso20022Plugin(UI(), {})
    parser = plugin.get_parser(os.path.join(SAMPLES_DIR, 'gcamp6.xml'))

    # WHEN
    with pytest.raises(exceptions.ParseError):
        stmt = parser.parse()


def test_parse_gcamp6():
    # GIVEN
    config = {
        "currency": "XXX"
    }
    plugin = Iso20022Plugin(UI(), config)

    parser = plugin.get_parser(os.path.join(SAMPLES_DIR, 'gcamp6.xml'))


    # WHEN
    stmt = parser.parse()

    # THEN
    assert stmt is not None

    assert stmt.account_id == 'CH2609000000924238861'
    assert stmt.currency is "XXX"

    assert stmt.bank_id is None
    assert stmt.end_balance == 10000.0
    assert stmt.end_date == datetime.datetime(2017, 1, 31, 0, 0)
    assert stmt.start_balance == 0.0
    assert stmt.start_date == datetime.datetime(2015, 12, 31, 0, 0)

    assert len(stmt.lines) == 5

    assert all(l.amount for l in stmt.lines)

    line0 = stmt.lines[0]

    assert line0.amount == 10000.0
    assert line0.memo == 'Account Transfer'
    assert line0.date == datetime.datetime(2016, 4, 23, 0, 0)
    assert line0.date_user == datetime.datetime(2016, 4, 23, 0, 0)
    assert line0.payee is None
    assert line0.refnum == '20160423000805545979476000000012'


def test_parse_davider80():
    # GIVEN
    config = {
        "currency": "CHF"
    }
    plugin = Iso20022Plugin(UI(), config)

    parser = plugin.get_parser(os.path.join(SAMPLES_DIR, 'davider80.xml'))

    # WHEN
    stmt = parser.parse()

    # THEN
    assert stmt is not None

    assert stmt.account_id == 'CHxxxxxxxxxxxxxxxxxxx'
    assert stmt.currency is "CHF"

    assert stmt.bank_id == "Raiffeisen"
    assert stmt.end_balance == 5753.46
    assert stmt.end_date == datetime.datetime(2017, 4, 11, 0, 0)
    assert stmt.start_balance == 9433.31
    assert stmt.start_date == datetime.datetime(2017, 4, 1, 0, 0)

    assert len(stmt.lines) == 16

    assert all(l.amount for l in stmt.lines)

    line0 = stmt.lines[0]

    assert line0.amount == -905.3
    assert line0.memo == 'Sistema di addebitamento diretto xxxxxxxxxxxxxxxxxxxxxx AG'
    assert line0.date == datetime.datetime(2017, 4, 3, 0, 0)
    assert line0.date_user == datetime.datetime(2017, 4, 1, 0, 0)
    assert line0.payee is None
    assert line0.refnum == '210564431020000000024556150000'


def test_unsupported():
    # GIVEN
    config = {
        "currency": "CHF"
    }
    plugin = Iso20022Plugin(UI(), config)

    # WHEN
    parser = plugin.get_parser(os.path.join(SAMPLES_DIR, 'unsupported.xml'))

    # THEN
    with pytest.raises(exceptions.ParseError):
        parser.parse()



"""Parser implementation for SEB generated statement reports"""

import re
import csv
import logging

from ofxstatement.parser import CsvStatementParser
from ofxstatement.plugin import Plugin
from ofxstatement.statement import Statement, StatementLine


class SebLV_CSVStatementParser(CsvStatementParser):
    date_format = "%d.%m.%Y"
    card_purchase_re = re.compile(r".*#(\d+)$")

    debug = (logging.getLogger().getEffectiveLevel() == logging.DEBUG)

    def split_records(self):
        csv_file = csv.reader(self.fin.readlines(), delimiter=';', quotechar='"')
        return csv_file

    def parse_record(self, line):
        if self.cur_record <= 2:
            # Skip header line
            return None

        if not self.statement.account_id:
            self.statement.account_id = line[16]

        # Get all fields
        type_code = line[12]
        date = line[1]
        date_user = line[11]
        c_or_d = line[14]
        amount = self.parse_float(line[3])
        id = line[10]
        refnum = line[10]
        note = line[9]
        payee_name = line[4]
        currency = line[17]

        # Convert LVL to EUR
        if currency == 'LVL':
            currency = 'EUR'
            amount = round(float(amount) / 0.702804, 2)

        # Create a statement line
        stmt_line = StatementLine(id, self.parse_datetime(date), note, self.parse_float(amount))
        stmt_line.payee = payee_name
        stmt_line.refnum = refnum
        stmt_line.date_user = self.parse_datetime(date_user)

        # Credit & Debit stuff
        stmt_line.trntype = "DEP"
        if c_or_d == 'D':
            stmt_line.amount = -stmt_line.amount
            stmt_line.trntype = "DEBIT"

        # Various types
        if 'PMNTCCRDCWDL' in type_code:
            stmt_line.trntype = "ATM"
        elif 'ACMTMDOPFEES' in type_code:
            stmt_line.trntype = "SRVCHG"
        elif 'LDASCSLNINTR' in type_code:
            stmt_line.trntype = "INT"
        elif 'PMNTCCRDOTHR' in type_code:
            stmt_line.trntype = "PAYMENT"
            m = self.card_purchase_re.match(stmt_line.memo)
            if m:
                # This is an electronic purchase. Extract check number from the memo field
                stmt_line.check_no = m.group(1)

        elif 'PMNTRCDTESCT' in type_code or 'PMNTICDTESCT' in type_code:
            stmt_line.trntype = "XFER"

        # DEBUG
        if self.debug:
            print(stmt_line, stmt_line.trntype)

        return stmt_line

    def parse_float(self, value):
        return value if isinstance(value, float) else float(value.replace(',', '.'))


class SebLVPlugin(Plugin):
    """Latvian SEB CSV"""

    def get_parser(self, fin):
        encoding = self.settings.get('charset', 'utf-8')
        f = open(fin, "r", encoding=encoding)
        parser = SebLV_CSVStatementParser(f)
        parser.statement.currency = self.settings.get('currency', 'EUR')
        return parser



from ofxstatement.parser import CsvStatementParser
from ofxstatement.plugin import Plugin
from ofxstatement import statement
import csv


class LbbAmazonCsvStatementParser(CsvStatementParser):
    mappings = {"date": 1, "memo": 3, "amount": 6}
    date_format = "%d.%m.%Y"
    ignore_amazon_points = False

    def split_records(self):
        return csv.reader(self.fin, delimiter=';')

    def parse_record(self, line):
        #Free Headerline
        if self.cur_record <= 1:
            return None

        # Empty transactions to include amazon points
        if line[6] == '':
            if self.ignore_amazon_points:
                return None 
            line[6] = '0,00'
        #Change decimalsign from , to .
        line[6] = line[6].replace(',', '.')

        # fill statement line according to mappings
        sl = super(LbbAmazonCsvStatementParser, self).parse_record(line)
        return sl


class LbbAmazonPlugin(Plugin):
    """
    Landensbank Berlin AG, Amazon Creditcard CSV
    """

    def get_parser(self, fin):
        f = open(fin, "r", encoding='iso-8859-1')
        parser = LbbAmazonCsvStatementParser(f)
        parser.statement.account_id = self.settings['account']
        parser.statement.currency = self.settings['currency']
        parser.statement.bank_id = self.settings.get('bank', 'LBB_Amazon')
        parser.ignore_amazon_points = self.settings.get('ignore_amazon_points', 'false').lower() in ('true', 'yes', 'on')
        return parser



from ofxstatement.parser import CsvStatementParser
from ofxstatement.plugin import Plugin
from ofxstatement import statement

DETAILS_FIELD = 5


class DanskeCsvStatementParser(CsvStatementParser):
    mappings = {"date": 0,
                "memo": 4,
                "amount": 8
                }
    date_format = "%Y:%m:%d"

    def parse(self):
        stmt = super(DanskeCsvStatementParser, self).parse()
        statement.recalculate_balance(stmt)
        return stmt

    def parse_record(self, line):
        if self.cur_record == 1:
            return None

        # fill statement line according to mappings
        sl = super(DanskeCsvStatementParser, self).parse_record(line)

        # generate transaction id out of available data
        sl.id = statement.generate_transaction_id(sl)
        if not sl.memo:
            sl.memo = line[DETAILS_FIELD]
        return sl

    def use_details_for_memo(self):
        self.mappings['memo'] = DETAILS_FIELD


class DanskePlugin(Plugin):
    """Lithuanian Danske bank CSV
    """

    def get_parser(self, fin):
        encoding = self.settings.get('charset', 'utf-8')
        f = open(fin, 'r', encoding=encoding)
        parser = DanskeCsvStatementParser(f)
        parser.statement.currency = self.settings['currency']
        parser.statement.account_id = self.settings['account']
        parser.statement.bank_id = self.settings.get('bank', 'Danske')
        if 'use-details-for-memo' in self.settings:
            parser.use_details_for_memo()
        return parser



import doctest

from ofxstatement.plugins.dkb_cc import DKBCCCsvStatementParser

def doctest_DKBCCCsvStatementParser():
    """Test DKBCCCsvStatementParser

    Open sample csv to parse
        >>> import os
        >>> csvfile = os.path.join(os.path.dirname(__file__),
        ...                        'samples', 'dkbcc.csv')

    Create parser object and parse:
        >>> fin = open(csvfile, 'r', encoding='iso-8859-1')
        >>> parser = DKBCCCsvStatementParser(fin)
        >>> statement = parser.parse()

    Check what we've got:
        >>> statement.account_id
        >>> len(statement.lines)
        7
        >>> statement.start_balance       
        >>> statement.start_date
        datetime.datetime(2012, 2, 7, 0, 0)
        >>> statement.end_balance
        -76.77
        >>> statement.end_date
        datetime.datetime(2013, 2, 4, 0, 0)
        >>> statement.currency
        'EUR'

    Check first line
        >>> l = statement.lines[0]
        >>> l.amount
        -95.0
        >>> l.payee 
        >>> l.memo
        'MY BANK FIL'
        >>> l.date
        datetime.datetime(2013, 2, 4, 0, 0)

    Check one more lines:
        >>> l=statement.lines[2]
        >>> l.amount
        0.21
        >>> l.payee
        >>> l.memo
        'HabenzinsenZ 000000057 T 031   0000'
        >>> l.date
        datetime.datetime(2013, 1, 23, 0, 0)

    """



def test_suite(*args):
    return doctest.DocTestSuite(optionflags=(doctest.NORMALIZE_WHITESPACE|
                                             doctest.ELLIPSIS|
                                             doctest.REPORT_ONLY_FIRST_FAILURE|
                                             doctest.REPORT_NDIFF
                                             ))
load_tests = test_suite



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

import os.path
import click
import logging

from ofxstatement.ofx import OfxWriter
from ofxstatement.plugins import paypal


@click.command()
@click.argument('path')
@click.option('--debug', is_flag=True, default=False)
def convert(path, debug):
    """Parse and print transactions from PayPal's *.csv files."""

    logging.basicConfig(level=logging.INFO, format='[%(levelname)s] %(message)s')

    root, ext = os.path.splitext(path)
    output_file = root + '.ofx'

    parser = paypal.PayPalPlugin(ui=None, settings=None).get_parser(path)
    statement = parser.parse()

    if debug:
        for line in statement.lines:
            print(line)
        return

    with open(output_file, 'w') as out:
        writer = OfxWriter(statement)
        out.write(writer.toxml())


if __name__ == '__main__':
    convert()



# -*- encoding: utf-8 -*-
import csv

from ofxstatement.plugin import Plugin
from .parser import NordeaCsvStatementParser, SIGNATURES


class NordeaDialect(csv.Dialect):
    """Definitions for reading information from Nordea account statements"""

    delimiter = "\t"
    doublequote = False
    escapechar = None
    lineterminator = "\r\n"
    quotechar = '"'
    quoting = csv.QUOTE_MINIMAL
    skipinitialspace = None


class NordeaPlugin(Plugin):
    """Nordea Suomi / Nordea Finland"""

    def get_csv_signature(self, csvfile):
        csvfile.readline()  # account # line
        csvfile.readline()  # empty line
        return csvfile.readline().strip()

    def get_parser(self, fin):
        f = open(fin, "r", encoding="utf-8")
        signature = self.get_csv_signature(f)
        if signature in SIGNATURES:
            parser = NordeaCsvStatementParser(f)
            parser.statement.account_id = self.settings['account']
            parser.statement.currency = self.settings['currency']
            parser.statement.bank_id = self.settings.get('bank', 'Nordea')
            return parser

        # no plugin with matching signature was found
        f.close()
        raise Exception(
            "No suitable Nordea parser found for this statement file."
        )



#!/usr/bin/python3

from distutils.core import setup

from setuptools import find_packages

version = "0.2.0.post1"

with open('README.rst') as f:
    long_description = f.read()

setup(name='ofxstatement-al_bank',
      version=version,
      author="Leonardo Brondani Schenkel",
      author_email="leonardo@schenkel.net",
      url="https://github.com/lbschenkel/ofxstatement-al_bank",
      description="Arbejdernes Landsbank plugin for ofxstatement",
      long_description=long_description,
      license="GPLv3",
      keywords=["ofx", "banking", "statement"],
      classifiers=[
          'Development Status :: 4 - Beta',
          'Programming Language :: Python :: 3',
          'Natural Language :: English',
          'Topic :: Office/Business :: Financial :: Accounting',
          'Topic :: Utilities',
          'Environment :: Console',
          'Operating System :: OS Independent',
          'License :: OSI Approved :: GNU General Public License v3 (GPLv3)'],
      packages=find_packages('src'),
      package_dir={'': 'src'},
      namespace_packages=["ofxstatement", "ofxstatement.plugins"],
      entry_points={
          'ofxstatement': ['al_bank = ofxstatement.plugins.al_bank:ALBankPlugin']
      },
      install_requires=['ofxstatement'],
      include_package_data=True,
      zip_safe=True
      )



#!/usr/bin/env python3
# This file is part of ofxstatement-austrian.
# See README.rst for more information.

import unittest

from ofxstatement.plugins.utils import \
    clean_multiple_whitespaces, fix_amount_string


class TestCleanMultipleWhiteSpaces(unittest.TestCase):
    """Unit tests for clean_multiple_whitespaces helper."""
    expected = "This is a test"

    def test_just_spaces(self):
        self.assertEqual(
            clean_multiple_whitespaces("This    is  a test"), self.expected)

    def test_just_tabs(self):
        self.assertEqual(
            clean_multiple_whitespaces("This	is	a	test"), self.expected)

    def test_mixed_tabs_and_spaces(self):
        self.assertEqual(
            clean_multiple_whitespaces(" This	is  a test  "), self.expected)

    def test_empty_string(self):
        self.assertEqual(clean_multiple_whitespaces(""), "")

    def test_string_with_spaces(self):
        self.assertEqual(clean_multiple_whitespaces("       "), "")


class TestFixAmountString(unittest.TestCase):
    """Unit tests for fix_amount_string helper."""

    def test_integer_string(self):
        self.assertEqual(fix_amount_string("11"), "11")

    def test_no_thousand_mark(self):
        self.assertEqual(fix_amount_string("1,23"), "1.23")

    def test_with_thousand_mark(self):
        self.assertEqual(fix_amount_string("100.234,23"), "100234.23")

# vim: tabstop=4 expandtab shiftwidth=4 softtabstop=4 smartindent autoindent



#!/usr/bin/env python3
# This file is part of ofxstatement-austrian.
# See README.rst for more information.

from setuptools import find_packages
from distutils.core import setup

version = "0.0.4"

with open("README.rst") as f:
    long_description = f.read()

setup(name="ofxstatement-austrian",
      version=version,
      author="Florian Preinstorfer",
      author_email="nblock@archlinux.us",
      url="https://github.com/nblock/ofxstatement-austrian",
      description=("Statement parser for several austrian banks"),
      long_description=long_description,
      license="GPLv3",
      keywords=[
          "ofxstatement", "easybank", "ing-diba",
          "livebank", "raiffeisen", "oberbank"],
      classifiers=[
          "Development Status :: 3 - Alpha",
          "Programming Language :: Python :: 3",
          "Programming Language :: Python :: 3.2",
          "Programming Language :: Python :: 3.3",
          "Programming Language :: Python :: 3.4",
          "Programming Language :: Python :: 3.5",
          "Natural Language :: English",
          "Topic :: Office/Business :: Financial :: Accounting",
          "Topic :: Utilities",
          "Environment :: Console",
          "Operating System :: OS Independent",
          "License :: OSI Approved :: GNU General Public License v3 (GPLv3)"],
      packages=find_packages("src"),
      package_dir={"": "src"},
      namespace_packages=["ofxstatement", "ofxstatement.plugins"],
      entry_points={
          "ofxstatement":
          [
              "easybank = ofxstatement.plugins.easybank:EasybankPlugin",
              "ing-diba = ofxstatement.plugins.ingdiba:IngDiBaPlugin",
              "livebank = ofxstatement.plugins.livebank:LivebankPlugin",
              "raiffeisen = ofxstatement.plugins.raiffeisen:RaiffeisenPlugin",
              "oberbank = ofxstatement.plugins.oberbank:OberbankPlugin"
          ]
      },
      install_requires=["ofxstatement"],
      test_suite="ofxstatement.plugins.tests",
      include_package_data=True,
      zip_safe=True
      )

# vim: tabstop=4 expandtab shiftwidth=4 softtabstop=4 smartindent autoindent



#!/usr/bin/env python3
# This file is part of ofxstatement-austrian.
# See README.rst for more information.

import csv
from ofxstatement import statement
from ofxstatement.parser import CsvStatementParser
from ofxstatement.plugin import Plugin
from ofxstatement.statement import generate_transaction_id
from ofxstatement.plugins.utils import \
    clean_multiple_whitespaces, fix_amount_string


class LivebankCsvParser(CsvStatementParser):
    """The csv parser for Livebank."""

    date_format = "%Y-%m-%d"

    mappings = {
        "date": 2,
        "amount": 7,
        "memo": 8,
        "payee": 9,
        }

    def parse(self):
        """Parse."""
        stmt = super(LivebankCsvParser, self).parse()
        statement.recalculate_balance(stmt)
        return stmt

    def split_records(self):
        """Split records using a custom dialect."""
        return csv.reader(self.fin, delimiter=";")

    def parse_record(self, line):
        """Parse a single record."""
        # Skip header line
        if self.cur_record == 1:
            return None

        # Skip lines without amount
        if line[7] == "0,00":
            return None

        # Account id
        if not self.statement.account_id:
            self.statement.account_id = line[0]

        # Currency
        if not self.statement.currency:
            self.statement.currency = line[6]

        # Cleanup parts
        line[7] = fix_amount_string(line[7])
        line[9] = clean_multiple_whitespaces(", ".join(line[9:]))

        # Create statement and fixup missing parts
        stmtline = super(LivebankCsvParser, self).parse_record(line)
        stmtline.trntype = 'DEBIT' if stmtline.amount < 0 else 'CREDIT'
        stmtline.id = generate_transaction_id(stmtline)

        return stmtline


class LivebankPlugin(Plugin):
    """Livebank (CSV)"""

    def get_parser(self, filename):
        """Get a parser instance."""
        encoding = self.settings.get('charset', 'iso-8859-1')
        f = open(filename, 'r', encoding=encoding)
        parser = LivebankCsvParser(f)
        parser.statement.bank_id = self.settings.get('bank', 'Livebank')
        return parser

# vim: tabstop=4 expandtab shiftwidth=4 softtabstop=4 smartindent autoindent



import os
import datetime

from ofxstatement.ui import UI
from ofxstatement.plugins.sberbank_txt import SberBankTxtPlugin


SAMPLES_DIR = os.path.join(os.path.dirname(__file__), 'samples')


def test_parse_maestro():
    plugin = SberBankTxtPlugin(UI(), {})
    s = plugin.get_parser(os.path.join(SAMPLES_DIR, 'sberbank_maestro.txt')).parse()

    assert s is not None

    assert s.account_id == 'СБЕРБАНК - MAESTRO XXXXXXXXX XXXX40696 ОСНОВНАЯ'
    assert s.currency == 'RUR'
    assert s.bank_id == 'SberBank'
    assert s.end_balance == 0
    assert s.start_balance == 0

    assert len(s.lines) == 4

    assert all(l.amount for l in s.lines)

    line0 = s.lines[0]

    assert line0.amount == 10000.0
    assert line0.memo == 'SBOL MOSCOW RUS'
    assert line0.date == datetime.datetime(2019, 3, 26, 0, 0)
    assert line0.trntype == 'DEBIT'


def test_parse_visa():
    plugin = SberBankTxtPlugin(UI(), {})
    s = plugin.get_parser(os.path.join(SAMPLES_DIR, 'sberbank_visa.txt')).parse()

    assert s is not None

    assert s.account_id == 'VISA GOLD XXXX XXXX XXX4 6122 ОСНОВНАЯ'
    assert s.currency == 'RUR'

    assert s.bank_id == 'SberBank'
    assert s.end_balance == 20877.29
    assert s.start_balance == 318.3

    assert len(s.lines) == 34

    assert all(l.amount for l in s.lines)

    assert s.lines[2].memo == 'ПОПОЛНЕНИЕ СЧЕТА'
    assert s.lines[2].trntype == 'DEBIT'

    assert s.lines[7].memo == 'PEREKRESTOK KRYLATSKOY E NOGINSK RU'
    assert s.lines[7].trntype == 'CREDIT'
    assert s.lines[7].amount == -1699.0
    assert s.lines[7].date == datetime.datetime(2018, 10, 4, 0, 0)

    assert s.lines[14].memo == 'TINKOFF BANK CARD2CARD Visa Direct RU'

    assert s.lines[25].memo == 'Rocketbank.ru Card2Car d MOSCOW RU'
    assert s.lines[25].trntype == 'CREDIT'

    assert abs(sum(l.amount for l in s.lines) + s.start_balance - s.end_balance) < 0.001



#!/usr/bin/env python3
# This file is part of ofxstatement-austrian.
# See README.rst for more information.

import datetime
import os
import unittest
from ofxstatement.statement import generate_transaction_id

from ofxstatement.plugins.oberbank import OberbankCsvParser


class TestOberbankCsvParser(unittest.TestCase):
    """Unit tests for OberbankCsvParser."""

    def setUp(self):
        csvfile = os.path.join(
            os.path.dirname(__file__), 'samples', 'oberbank.csv')
        with open(csvfile, 'r', encoding='cp1252') as fin:
            self.statement = OberbankCsvParser(fin).parse()

    def test_statement_properties(self):
        self.assertEqual(len(self.statement.lines), 5)
        self.assertEqual(self.statement.start_balance, 0.0)
        self.assertAlmostEqual(self.statement.end_balance, -9.0)
        self.assertEqual(self.statement.currency, "EUR")
        self.assertEqual(
            self.statement.start_date, datetime.datetime(2017, 3, 15, 0, 0))
        self.assertEqual(
            self.statement.end_date, datetime.datetime(2017, 3, 15, 0, 0))

    def test_line0_payment_description_reference(self):
        line = self.statement.lines[0]
        self.assertEqual(line.amount, -11.0)
        self.assertEqual(
            line.memo,
            "Zahlungsreferenz, Empfängername, Adresszeile1, Adresszeile2")
        self.assertEqual(line.trntype, "DEBIT")
        self.assertEqual(line.date, datetime.datetime(2017, 3, 15, 0, 0))
        self.assertEqual(line.id, generate_transaction_id(line))

    def test_line1_payment_description_usage(self):
        line = self.statement.lines[1]
        self.assertEqual(line.amount, -10.0)
        self.assertEqual(
            line.memo,
            "Verwendungszweck, Empfängername, Adresszeile1, Adresszeile2, "
            "Verwendungszweck, VerwendungszweckZeile2, "
            "VerwendungszweckZeile3, VerwendungszweckZeile4")
        self.assertEqual(line.trntype, "DEBIT")
        self.assertEqual(line.date, datetime.datetime(2017, 3, 15, 0, 0))
        self.assertEqual(line.id, generate_transaction_id(line))

    def test_line2_payment_description_end2end(self):
        line = self.statement.lines[2]
        self.assertEqual(line.amount, -9.0)
        self.assertEqual(
            line.memo,
            "End2EndID, Empfängername, Adresszeile1, Adresszeile2")
        self.assertEqual(line.trntype, "DEBIT")
        self.assertEqual(line.date, datetime.datetime(2017, 3, 15, 0, 0))
        self.assertEqual(line.id, generate_transaction_id(line))

    def test_line3_credeit(self):
        line = self.statement.lines[3]
        self.assertEqual(line.amount, 11.0)
        self.assertEqual(line.memo, "Empfängername, SCOR, Verwendungszweck")
        self.assertEqual(line.trntype, "CREDIT")
        self.assertEqual(line.date, datetime.datetime(2017, 3, 15, 0, 0))
        self.assertEqual(line.id, generate_transaction_id(line))

    def test_line4_credeit(self):
        line = self.statement.lines[4]
        self.assertEqual(line.amount, 10.0)
        self.assertEqual(line.memo, "Empfängername, SCOR, Verwendungszweck")
        self.assertEqual(line.trntype, "CREDIT")
        self.assertEqual(line.date, datetime.datetime(2017, 3, 15, 0, 0))
        self.assertEqual(line.id, generate_transaction_id(line))

# vim: tabstop=4 expandtab shiftwidth=4 softtabstop=4 smartindent autoindent



# coding=utf-8

"""
Tests for the ofxstatement Betterment plugin.
"""
from datetime import date
import logging
import unittest
from ofxstatement.statement import StatementLine

import os

from ofxstatement.plugins.betterment import BettermentPlugin, is_zero, generate_stable_transaction_id

__author__ = 'cmayes'

logging.basicConfig(level=logging.DEBUG)
logger = logging.getLogger('test_betterment')

DATA_DIR = os.path.join(os.path.dirname(__file__), 'test_data')
ZERO_BAL_CSV_PATH = os.path.join(DATA_DIR, 'transactions.csv')
POS_CSV_PATH = os.path.join(DATA_DIR, 'transactions_positive.csv')
NOV_2015_CSV_PATH = os.path.join(DATA_DIR, 'transactions_11_2015.csv')
NOV_2015_CSV_PATH_NO_COMPLETED = os.path.join(DATA_DIR, 'transactions_11_2015_no_completed.csv')
ALL_ZERO_CSV_PATH = os.path.join(DATA_DIR, 'transactions_zeros.csv')
HEADER_CSV_PATH = os.path.join(DATA_DIR, 'transactions_header.csv')
EMPTY_CSV_PATH = os.path.join(DATA_DIR, 'transactions_empty.csv')
BLANK_BAL_PATH = os.path.join(DATA_DIR, 'transactions_blankbal.csv')


# Shared Methods #


class TestBettermentStatement(unittest.TestCase):
    def testZeroBalance(self):
        """Statement starts with a zero balance."""
        stmt = BettermentPlugin(None, {}).get_parser(ZERO_BAL_CSV_PATH).parse()
        self.assertAlmostEqual(0.0, stmt.start_balance)
        self.assertAlmostEqual(817.44, stmt.end_balance)
        self.assertEqual(date(2015, 9, 22), to_date(stmt.start_date))
        self.assertEqual(date(2015, 9, 25), to_date(stmt.end_date))
        self.assertEqual(5, len(stmt.lines))
        for stline in stmt.lines:
            self.assertTrue(stline.memo in ('Initial Deposit from ****', 'Market Change'))
            self.assertFalse(is_zero(stline.amount))
            self.assertTrue(date(2015, 9, 25) >= to_date(stline.date) >= date(2015, 9, 22))

    def testPositiveBalance(self):
        """Statement starts with a positive balance."""
        stmt = BettermentPlugin(None, {}).get_parser(POS_CSV_PATH).parse()
        self.assertAlmostEqual(1000.0, stmt.start_balance)
        self.assertAlmostEqual(1817.44, stmt.end_balance)
        self.assertEqual(date(2015, 9, 22), to_date(stmt.start_date))
        self.assertEqual(date(2015, 9, 25), to_date(stmt.end_date))
        self.assertEqual(5, len(stmt.lines))
        for stline in stmt.lines:
            self.assertTrue(stline.memo in ('Initial Deposit from ****', 'Market Change'))
            self.assertFalse(is_zero(stline.amount))
            self.assertTrue(date(2015, 9, 25) >= to_date(stline.date) >= date(2015, 9, 22))

    def testPositiveBalanceNov2015(self):
        """Statement starts with a positive balance and the new November 2015 CSV format."""
        stmt = BettermentPlugin(None, {}).get_parser(NOV_2015_CSV_PATH).parse()
        self.assertAlmostEqual(1631.25, stmt.start_balance)
        self.assertAlmostEqual(1254.43, stmt.end_balance)
        self.assertEqual(date(2015, 10, 29), to_date(stmt.start_date))
        self.assertEqual(date(2015, 11, 11), to_date(stmt.end_date))
        self.assertEqual(12, len(stmt.lines))
        for stline in stmt.lines:
            self.assertTrue(stline.memo in ('Initial Deposit from ****', 'Market Change', 'Dividend'))
            self.assertFalse(is_zero(stline.amount))
            self.assertTrue(date(2015, 11, 11) >= to_date(stline.date) >= date(2015, 10, 29))

    def testPositiveBalanceNov2015NoCompleted(self):
        """Statement is missing a completed date for the latest line in the new November 2015 CSV format."""
        stmt = BettermentPlugin(None, {}).get_parser(NOV_2015_CSV_PATH_NO_COMPLETED).parse()
        self.assertAlmostEqual(1631.25, stmt.start_balance)
        self.assertAlmostEqual(1240.33, stmt.end_balance)
        self.assertEqual(date(2015, 10, 29), to_date(stmt.start_date))
        self.assertEqual(date(2015, 11, 10), to_date(stmt.end_date))
        self.assertEqual(11, len(stmt.lines))
        for stline in stmt.lines:
            self.assertTrue(stline.memo in ('Initial Deposit from ****', 'Market Change', 'Dividend'))
            self.assertFalse(is_zero(stline.amount))
            self.assertTrue(date(2015, 11, 10) >= to_date(stline.date) >= date(2015, 10, 29))

class TestBettermentSettings(unittest.TestCase):
    def testAccountNoFilterZeros(self):
        """Set an account and disable zero-value transaction filtering."""
        stmt = BettermentPlugin(None, {'account': '12345', 'filter_zeros': 'false'}
                                ).get_parser(ZERO_BAL_CSV_PATH).parse()
        self.assertEqual(stmt.account_id, '12345')
        self.assertEqual(stmt.bank_id, 'Betterment')
        # Check that the zero balance lines are present
        self.assertEqual(7, len(stmt.lines))

    def testBankNoFilterZeros(self):
        """Set a custom bank and filter zero-value transactions."""
        stmt = BettermentPlugin(None, {'bank': 'CustomBank', 'filter_zeros': 'true'}
                                ).get_parser(ZERO_BAL_CSV_PATH).parse()
        self.assertIsNone(stmt.account_id)
        self.assertEqual(stmt.bank_id, 'CustomBank')
        # Check that the zero balance lines are present
        self.assertEqual(5, len(stmt.lines))


class TestBettermentEmpties(unittest.TestCase):
    def testAccountFilterZerosAll(self):
        """All lines filtered by zeros policy."""
        stmt = BettermentPlugin(None, {}).get_parser(ALL_ZERO_CSV_PATH).parse()
        self.assertIsNone(stmt.start_balance)
        self.assertIsNone(stmt.end_balance)
        self.assertIsNone(stmt.start_date)
        self.assertIsNone(stmt.end_date)
        self.assertEqual(0, len(stmt.lines))

    def testHeaderOnly(self):
        """File only has a header line."""
        stmt = BettermentPlugin(None, {}).get_parser(HEADER_CSV_PATH).parse()
        self.assertIsNone(stmt.start_balance)
        self.assertIsNone(stmt.end_balance)
        self.assertIsNone(stmt.start_date)
        self.assertIsNone(stmt.end_date)
        self.assertEqual(0, len(stmt.lines))

    def testEmpty(self):
        """Empty file."""
        stmt = BettermentPlugin(None, {}).get_parser(EMPTY_CSV_PATH).parse()
        self.assertIsNone(stmt.start_balance)
        self.assertIsNone(stmt.end_balance)
        self.assertIsNone(stmt.start_date)
        self.assertIsNone(stmt.end_date)
        self.assertEqual(0, len(stmt.lines))

    def testEmptyEndingBalance(self):
        """Strip out pending transactions."""
        stmt = BettermentPlugin(None, {}).get_parser(BLANK_BAL_PATH).parse()
        self.assertAlmostEqual(1319.74, stmt.start_balance)
        self.assertAlmostEqual(1692.68, stmt.end_balance)
        self.assertEqual(date(2015, 10, 2), to_date(stmt.start_date))
        self.assertEqual(date(2015, 10, 2), to_date(stmt.end_date))
        self.assertEqual(3, len(stmt.lines))
        for stline in stmt.lines:
            self.assertTrue(stline.memo in ('Dividend Reinvestment', 'Market Change'))
            self.assertFalse(is_zero(stline.amount))
            self.assertEqual(date(2015, 10, 2), to_date(stline.date))


class TestBettermentStatementIdStability(unittest.TestCase):
    all_hash = "316fcd14b9bb7d9bdb6a00473356c896aede98da3df9268bc57c319f209ad3d1"
    amt_hash = "4ebc4a141b378980461430980948a55988fbf56f85d084ac33d8a8f61b9fab88"
    none_hash = "e3b0c44298fc1c149afbf4c8996fb92427ae41e4649b934ca495991b7852b855"

    def testAllFilled(self):
        """All of the fields used in the hash have a value"""
        sline = StatementLine()
        sline.date = date(2015, 10, 2)
        sline.memo = "Test statement line"
        sline.amount = 123.45
        self.assertEqual(self.all_hash, generate_stable_transaction_id(sline))

    def testAmount(self):
        """Only amount is set"""
        sline = StatementLine()
        sline.amount = 123.45
        self.assertEqual(self.amt_hash, generate_stable_transaction_id(sline))

    def testNone(self):
        """Nothing is set"""
        sline = StatementLine()
        self.assertEqual(self.none_hash, generate_stable_transaction_id(sline))


def to_date(dt):
    """Creates a comparable date object from the given datetime."""
    return date(dt.year, dt.month, dt.day)



import doctest

from ofxstatement.plugins.alfabank import AlfabankStatementParser

def doctest_AlfaParser():
    """Test generic CsvStatementParser

    Lets define alfabank sample csv to parse and write it to file-like object

        >>> import io
        >>> csv = '''
        ... Мой счет;11111111111111111111;RUR;31.05.17;ABCDEF11111111111;{VO11111} Перечисление ден. средств (зарплата за май 2017 г.);33123,56;0;
        ... Мой счет;11111111111111111111;RUR;03.05.17;ABCDEF22222222222;Комиссия за переводы в рублях за период с1МАЙ17 по 1МАЙ17 поруч.поданы ч/з ИнтернетБанк Согл.тариф.Банка Иванов Иван Иванович;0;50;
        ... Мой счет;11111111111111111111;RUR;03.05.17;ABC_1AA11A;123456++++++6789    11111111\\\\123\\\\Visa Direct\\\\SOME BANK             03.05.17 01.05.17    1500.00  RUR MCC1234;0;1500;
        ... '''
        >>> f = io.StringIO(csv)

    Create and configure csv parser:
        >>> parser = AlfabankStatementParser(f)
        >>> parser.statement.bank_id = 'Alfabank'

    And parse csv:
        >>> statement = parser.parse()
        >>> statement.account_id
        '11111111111111111111'
        >>> statement.currency
        'RUR'
        >>> len(statement.lines)
        3
        >>> statement.lines[0].amount
        Decimal('33123.56')
        >>> statement.lines[0].memo
        '{VO11111} Перечисление ден. средств (зарплата за май 2017 г.)'
        >>> statement.lines[0].refnum
        'ABCDEF11111111111'
        >>> statement.lines[0].date
        datetime.datetime(2017, 5, 31, 0, 0)
        >>> statement.lines[0].trntype
        'DEBIT'

        >>> statement.lines[1].amount
        Decimal('-50')
        >>> statement.lines[1].date
        datetime.datetime(2017, 5, 3, 0, 0)
        >>> statement.lines[1].memo
        'Комиссия за переводы в рублях за период с1МАЙ17 по 1МАЙ17 поруч.поданы ч/з ИнтернетБанк Согл.тариф.Банка Иванов Иван Иванович'
        >>> statement.lines[1].trntype
        'SRVCHG'

        >>> statement.lines[2].amount
        Decimal('-1500')
        >>> statement.lines[2].memo
        '123456++++++6789    11111111\\\\123\\\\Visa Direct\\\\SOME BANK             03.05.17 01.05.17    1500.00  RUR MCC1234'
        >>> statement.lines[2].date
        datetime.datetime(2017, 5, 3, 0, 0)
        >>> statement.lines[2].trntype
        'CREDIT'

    """

def doctest_AlfaParser_userdates():
    """Test generic CsvStatementParser

    Lets define alfabank sample csv to parse and write it to file-like object

        >>> import io
        >>> csv = '''
        ... Мой счет;11111111111111111111;RUR;03.05.17;ABC_1AA11A;123456++++++6789    11111111\\\\123\\\\Visa Direct\\\\SOME BANK             03.05.17 01.05.17    1500.00  RUR MCC1234;0;1500;
        ... '''
        >>> f = io.StringIO(csv)

    Create and configure csv parser:
        >>> parser = AlfabankStatementParser(f)
        >>> parser.statement.bank_id = 'Alfabank'
        >>> parser.user_date = True

    And parse csv:
        >>> statement = parser.parse()
        >>> len(statement.lines)
        1
        >>> statement.lines[0].amount
        Decimal('-1500')
        >>> statement.lines[0].memo
        '123456++++++6789    11111111\\\\123\\\\Visa Direct\\\\SOME BANK             03.05.17 01.05.17    1500.00  RUR MCC1234'
        >>> statement.lines[0].date
        datetime.datetime(2017, 5, 1, 0, 0)
    """


def test_suite(*args):
    return doctest.DocTestSuite(optionflags=(doctest.NORMALIZE_WHITESPACE|
                                             doctest.ELLIPSIS|
                                             doctest.REPORT_ONLY_FIRST_FAILURE|
                                             doctest.REPORT_NDIFF
                                             ))


#    Tinkoff Bank (http://tinkoff.ru) plugin for ofxstatement
#
#    Copyright 2013 Andrey Lebedev <andrey@lebedev.lt>
#    Copyright 2016 Alexander Gerasiov <gq@cs.msu.su>
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License version 3 as
#    published by the Free Software Foundation.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.

from ofxstatement.parser import StatementParser
from ofxstatement.plugin import Plugin
from ofxstatement import statement
from datetime import datetime
import csv

# file format options
t_delimiter = ';'
t_time_format = '%d.%m.%Y %H:%M:%S'
t_encoding = 'cp1251'
t_fieldnames = ['op_time', 'tr_time', 'card', 'status', 'op_amount', 'op_currency', 'amount',
                'currency', 'cashback', 'category', 'MCC', 'description', 'bonus']
t_type_map = {
    u"Капитализация": 'DIV',
    u"Вознаграждение за операции покупок": 'DIV',
    u"Пополнение. Тинькофф Банк. Бонус": 'DIV',
    u"Проценты на остаток по счету": 'DIV',
    u"Плата за обслуживание": 'SRVCHG',
    u"Комиссия за выдачу наличных": 'SRVCHG',
    u"Снятие наличных": 'ATM',
    u"Пополнение": 'XFER',
    u"На счет в другом банке": 'XFER',
    u"Перевод c карты другого банка": 'XFER',
    u"Изъятие вклада при закрытии.": 'XFER',
    u"Внешний банковский перевод": 'XFER',
}


def parse_type(type, amount):
    for filter in t_type_map.keys():
        if type.startswith(filter):
            return t_type_map[filter]

    result = None

    if amount > 0:
        result = 'DEBIT'
    elif amount < 0:
        result = 'CREDIT'

    # print("Unknown type \"%s\", consider %s"%(type, result))
    return result


class TinkoffStatementParser(StatementParser):
    statement = None

    def __init__(self, fin):
        self.statement = statement.Statement()
        self.fin = fin
        # Skip 1st row with column's headers
        self.fin.readline()
        self.cur_record = 1

    def split_records(self):
        return csv.DictReader(self.fin, delimiter=t_delimiter, fieldnames=t_fieldnames)

    def parse_record(self, line):
        transaction = statement.StatementLine()

        if not line['status'] == 'OK':
            print("Notice: Skipping line %d: Transaction time %s status is %s." % (
            self.cur_record, line['op_time'], line['status']))
            return None

        if not self.statement.currency:
            self.statement.currency = line['currency']

        if not line['currency'] == self.statement.currency:
            print("Transaction %s currency '%s' differ from account currency '%s'." % (
            line['op_time'], line['currency'], self.statement.currency))
            return None

        transaction.date = datetime.strptime(line['op_time'], t_time_format)

        transaction.amount = float(line['amount'].replace(',', '.'))

        transaction.trntype = parse_type(line['description'], transaction.amount)

        transaction.memo = "%s: %s" % (line['category'], line['description'])

        if line['MCC']:
            transaction.memo = "%s, %s" % (transaction.memo, line['MCC'])

        if line['card']:
            transaction.memo = "%s, %s" % (transaction.memo, line['card'])

        if transaction.trntype:
            return transaction
        else:
            return None


class TinkoffPlugin(Plugin):
    """Tinkoff Bank CSV (http://tinkoff.ru)
    """

    def get_parser(self, fin):
        f = open(fin, 'r', encoding=t_encoding)
        parser = TinkoffStatementParser(f)
        parser.statement.currency = self.settings.get('currency')
        parser.statement.account_id = self.settings['account']
        parser.statement.bank_id = self.settings.get('bank', 'Tinkoff')
        return parser



import doctest

from ofxstatement.plugins.swedbank import SwedbankCsvStatementParser

def doctest_SwedbankCsvStatementParser():
    """Test SwedbankCsvStatementParser

    Open sample csv to parse
        >>> import os
        >>> csvfile = os.path.join(os.path.dirname(__file__),
        ...                        'samples', 'swedbank.csv')

    Create parser object and parse:
        >>> fin = open(csvfile, 'r', encoding='utf-8')
        >>> parser = SwedbankCsvStatementParser(fin)
        >>> parser.statement.currency = 'LTL'
        >>> statement = parser.parse()

    Check what we've got:
        >>> statement.account_id
        'LT797300010XXXXXXXXX-LTL'
        >>> len(statement.lines)
        5
        >>> statement.start_balance
        2123.82
        >>> statement.start_date
        datetime.datetime(2012, 1, 1, 0, 0)
        >>> statement.end_balance
        3917.3
        >>> statement.end_date
        datetime.datetime(2012, 1, 31, 0, 0)
        >>> statement.currency
        'LTL'

    Check first line
        >>> l = statement.lines[0]
        >>> l.amount
        -14.34
        >>> l.payee
        "McDonald's restoranas AKR Vilnius"
        >>> l.memo
        "PIRKINYS ... 00000"
        >>> l.id
        '2012010200041787'
        >>> l.check_no
        '059553'
        >>> l.date
        datetime.datetime(2012, 1, 2, 0, 0)
        >>> l.date_user
        datetime.datetime(2011, 12, 30, 0, 0)

    Check line with awkward quotation marks:
        >>> l = statement.lines[2]
        >>> l.id
        '2012012600096815'
        >>> l.amount
        -12.2
        >>> l.payee
        'UAB "Naktida"'
        >>> l.memo
        'PIRKINYS 0000000000000000 ... UAB "Naktida" ... 00000'

    Check income line:
        >>> l = statement.lines[3]
        >>> l.id
        '2012011000673562'
        >>> l.amount
        1600.0
        >>> l.payee
        'Company'
        >>> l.memo
        'Salary'

    Check line with empty payee:
        >>> l = statement.lines[4]
        >>> l.id
        '2012022900875660'
        >>> l.payee
        ''


    """



def test_suite(*args):
    return doctest.DocTestSuite(optionflags=(doctest.NORMALIZE_WHITESPACE|
                                             doctest.ELLIPSIS|
                                             doctest.REPORT_ONLY_FIRST_FAILURE|
                                             doctest.REPORT_NDIFF
                                             ))
load_tests = test_suite



from ofxstatement.plugin import Plugin
from ofxstatement.parser import CsvStatementParser
from ofxstatement.statement import StatementLine
import csv


class ConsorsParser(CsvStatementParser):
    
    # 0 Buchung;
    # 1 Valuta 
    # 2 Sender / Empfänger
    # 3 IBAN / Konto-Nr.
    # 4 BIC / BLZ 
    # 5 Buchungstext 
    # 6 Verwendungszweck 
    # 7 Betrag in EUR

    mappings = {"date": 1, "amount": 7}
    date_format = "%d.%m.%Y"

    def split_records(self):
        return csv.reader(self.fin, delimiter=';')

    def parse_record(self, line):

        if self.cur_record == 1:
            self.statement.currency = line[7].strip('Betrag in ')
            return None

        # Remove dots (German decimal point handling)
        # 2.000,00 => 2000,00
        line[7] = line[7].replace('.', '')

        # Replace comma with dot (German decimal point handling)
        # 2000.00 => 2000.00
        line[7] = line[7].replace(',', '.')

        # fill statement line according to mappings
        sl = super(ConsorsParser, self).parse_record(line)

        sl.memo = line[5] + "; " + line[6] + "; " + line[2]
        return sl
 

class ConsorsPlugin(Plugin):

    def get_parser(self, filename):
        f = open(filename, "r", encoding='utf-8')
        parser = ConsorsParser(f)
        parser.statement.account_id = "Consorsbank-Konto"
        parser.statement.bank_id = "Consorsbank"
        return parser




"""Parser implementation for swedbank generated statement reports"""

import re

from ofxstatement.parser import CsvStatementParser
from ofxstatement.plugin import Plugin

LINETYPE_TRANSACTION = "20"
LINETYPE_STARTBALANCE = "10"
LINETYPE_ENDBALANCE = "86"

CARD_PURCHASE_RE = re.compile(
    r"PIRKINYS \d+ (\d\d\d\d\.\d\d\.\d\d) .* \((\d+)\).*")


class SwedbankCsvStatementParser(CsvStatementParser):
    mappings = {"date": 2,
                "payee": 3,
                "memo": 4,
                "amount": 5,
                "id": 8}

    def split_records(self):
        # We cannot parse swedbank csv as regular csv because swedbanks format
        # uses unescaped quote symbols.
        return self.fin.readlines()

    def parse_record(self, line):
        if not line.strip():
            return None

        if self.cur_record == 1:
            # Skip header line
            return None

        # Split line to the parts and strip quotes around fields
        parts = [l[1:] for l in line.split('",')]
        if not self.statement.account_id:
            self.statement.account_id = \
                "%s-%s" % (parts[0], self.statement.currency)

        lineType = parts[1]

        if lineType == LINETYPE_TRANSACTION:
            # parse transaction line in standard fasion
            stmtline = super(SwedbankCsvStatementParser, self).parse_record(parts)
            if parts[7] == "D":
                stmtline.amount = -stmtline.amount

            linecur = parts[6]
            if self.statement.currency != linecur:
                # Skip lines with different currency
                return None

            m = CARD_PURCHASE_RE.match(stmtline.memo)
            if m:
                # this is an electronic purchase. extract some useful
                # information from memo field
                dt = m.group(1).replace(".", "-")
                stmtline.date_user = self.parse_datetime(dt)
                stmtline.check_no = m.group(2)

            return stmtline

        elif lineType == LINETYPE_ENDBALANCE:
            self.statement.end_balance = self.parse_float(parts[5])
            self.statement.end_date = self.parse_datetime(parts[2])

        elif lineType == LINETYPE_STARTBALANCE:
            self.statement.start_balance = self.parse_float(parts[5])
            self.statement.start_date = self.parse_datetime(parts[2])


class SwedbankPlugin(Plugin):
    """Lithuanian Swedbank CSV"""

    def get_parser(self, fin):
        f = open(fin, "r")
        parser = SwedbankCsvStatementParser(f)
        parser.statement.currency = self.settings.get('currency', 'EUR')
        return parser



import csv

from ofxstatement.plugin import Plugin
from ofxstatement.statement import Statement, StatementLine, BankAccount
from ofxstatement.parser import CsvStatementParser


class OtpCreditPlugin(Plugin):
    """OTP Credit Card Statements (CSV)
    """

    def get_parser(self, filename):
        return OtpCreditParser(filename)


class OtpCreditParser(CsvStatementParser):

    date_format = "%Y%m%d"
    encoding = 'utf-8'

    mappings = {
        "date": 5,
        "payee": 10,
        "memo": 9,
        "amount": 2,
        "trntype": 1,
    }

    def __init__(self, filename):
        self.statement = Statement(
            bank_id = "OTP Bank Nyrt.",
            account_id = self.get_account_id_from_file(filename)
        )
        self.fin = open(filename, encoding=self.encoding)

    def get_account_id_from_file(self, filename):
        with open(filename, encoding=self.encoding) as csvfile:
            first_line = next(self._get_reader(csvfile))
            return first_line[0]

    def split_records(self):
        return self._get_reader(self.fin)

    def _get_reader(self, fin):
        return csv.reader(fin, delimiter=';')

    def parse_value(self, value, field):
        if field == "trntype":
            return self.parse_trntype(value)
        elif field == "memo":
            return self.parse_memo(value)
        else:
            return super().parse_value(value, field)

    def parse_trntype(self, value):
        transaction_mapping = {
            "T": "CREDIT",
            "J": "DEBIT"
        }
        return transaction_mapping[value]

    def parse_memo(self, value):
        if value == "VÁSÁRLÁS KÁRTYÁVAL":
            # this is completely redundant and non-informative, let's just drop it
            return None
        else:
            return value


"""Parser for the statement the Czech semi-formal standard GPC
(used by FIO, among others)"""
import logging
from decimal import Decimal

from ofxstatement.parser import StatementParser
from ofxstatement.plugin import Plugin
from ofxstatement.statement import Statement, StatementLine

log = logging.getLogger('GPC')


class GPCParser(StatementParser):
    date_format = '%d%m%y'

    def __init__(self, inf, encoding="windows-1250", bank_id='2010'):
        super(GPCParser, self).__init__()
        log.debug('inf = {}'.format(inf))
        self.input = inf
        self.enc = encoding
        # Defaults to the bank_id of FIO banka, a.s.
        self.statement = Statement(bank_id=bank_id, currency='CZK')

    @staticmethod
    def get_acc_type(instr):
        if instr == ord('1'):
            return 'DEBIT'
        elif instr == ord('2'):
            return 'CREDIT'
        # OFX doesn't seem to be able to recognize storno transactions.
        #elif instr == ord('4'):
        #    return ACC_T.STORNO_DEBET
        #elif instr == ord('5'):
        #    return ACC_T.STORNO_KREDIT
        else:
            raise ValueError(
                'Unknown type of transation account: {}'.format(instr))

    @staticmethod
    def get_money(instr, signstr, pluschar='+'):
        instr = instr.decode().lstrip('0')
        if instr == '':
            instr = '0'
        num = Decimal(instr) / Decimal(100)
        if signstr == ord('-'):
            num = -num
        elif signstr != ord(pluschar):
            raise ValueError('Unknown sign "{}"'.format(signstr))
        return num

    def split_records(self):
        """Return iterable object consisting of a line per transaction
        """
        while True:
            res = self.input.read(130)
            if len(res) == 0:
                break
            yield res

    @staticmethod
    def __generate_statement_line(tr):
        memo_str = ''
        stat_line = StatementLine(id=tr['tr_statmt_no'],
                                  date=tr['maturity_date'],
                                  memo='',
                                  amount=tr['amount'])

        if len(tr['other_acc_no']) > 0:
            memo_str += 'For acc# {}'.format(tr['other_acc_no'])
        if len(tr['bank_code']) > 0:
            memo_str += '/{}'.format(tr['bank_code'])
        if len(tr['variable_symbol']) > 0:
            memo_str += ', VS: {}'.format(tr['variable_symbol'])
        if len(tr['specific_symbol']) > 0:
            memo_str += ', SS: {}'.format(tr['specific_symbol'])
        if len(tr['client_name']) > 0:
            memo_str += ', {}'.format(tr['client_name'])

        if len(memo_str) > 0:
            stat_line.memo = memo_str

        return stat_line

    def parse_record(self, line):
        """
        Each record is fixed-length (130 bytes) record type. There are
        two types of records distinguished by the first three bytes. If
        these are '074' then it is the whole statement record, if '075'
        then it is a transaction record.

        Statement record has this format:

            1-3     "074 " = type of record as "Data - výpis v Kč"
                    ("Data - statement in CZK")
            4-19    the account for which the statement is provided,
                    left-padded by zeros.
            20-39   20 alphanumeric characters of the shortened account name,
                    right-padded by spaces
            40-45   date of the previous statament in forma DDMMYY
            46-59   previous balance in hellers (1/100 CZK), 14 numeric
                    characters left-padded by zeros
            60      sign of previous balance, 1 character "+" or "-"
            61-74   current balance in hellers (1/100 CZK), 14 numeric
                    characters left-padded by zeros
            75      sign of the current balance, 1 character "+" or "-"
            76-89   amount of debet (MD) in hellers (1/100 CZK), 14
                    numeric characters left-padded by zeros
            90      sign of the debet amount (MD), 1 character "0" or "-"
            91-104  amount of kredit (D) in hellers (1/100 CZK), 14
                    numeric characters left-padded by zeros
            105     sign of the credit amount (D), 1 character "0" or "-"
            106-108 statement number
            109-114 statement date in format DDMMYY
            115-128 (filled by 14 spaces to make records of the same length)
            129-130 ending characters CR and LF

        Transaction record has this format:

            1-3     "075 " = type of record as "Data - obratová položka"
                    ("Data - transaction record")
            4-19    assigned account number 16 numeric characters
                    left-padded by zeros
            20–35   account number 16 numeric characters s vodící minulami
                    (or possibly prefix + account number)
            36–48   number of the transaction statement, 13 numeric characters
            49–60   amount in hellers (1/100 CZK) 12 numeric characters
                    left-padded by zeros
            61      accounting code in relation to the account number:
                    1 = debit transaction
                    2 = credit transaction
                    4 = storno of the debit transaction
                    5 = storno of the credit transaction
            62–71   variabile symbol 10 numeric characters left-padded by zeros
            72–81   constant symbol 10 numeric characters left-padded by
                    zeros in form BBBBKSYM, where:
                    BBBB = bank code
                    KSYM = constant symbol
            82–91   specific symbol 10 numeric characters left-padded by zeros
            92–97   "000000" = valuta
            98–117  20 alphanumeric characters shortened client name,
                    if necessary right-padded by spaces
            118     "0"
            119-122 "0203" = currency code for CZK
            123-128 maturity date in format DDMMYY
            129-130 ending characters CR and LF


        """
        assert len(line) == 130
        log.debug('line = {}'.format(line))

        act = {}
        trans = []
        typ = line[0:3]
        if typ == b'074':
            act['acc_no'] = line[3:19].decode().lstrip('0')
            act['acc_name'] = line[19:39].decode(self.enc).rstrip()
            act['orig_date'] = self.parse_datetime(line[39:45].decode())
            act['orig_amount'] = self.get_money(line[45:59], line[59])
            act['new_amount'] = self.get_money(line[60:74], line[74])
            act['debet_amount'] = self.get_money(line[75:89], line[89], '0')
            act['kredit_amount'] = self.get_money(line[90:104], line[104], '0')
            act['statement_no'] = int(line[105:108])
            act['statement_date'] = self.parse_datetime(line[108:114].decode())
        elif typ == b'075':
            tr = {}

            tr['acc_no'] = line[3:19].decode().lstrip('0')
            tr['other_acc_no'] = line[19:35].decode().lstrip('0')
            tr['tr_statmt_no'] = line[35:48].decode().lstrip('0')
            tr['amount'] = self.get_money(line[48:60], ord('+'))
            tr['other_acc_type'] = self.get_acc_type(line[60])
            tr['variable_symbol'] = line[61:71].decode().lstrip('0')
            var_s_bank_code = line[71:81].decode().lstrip('0')
            tr['bank_code'] = var_s_bank_code[0:4]
            tr['constant_symbol'] = var_s_bank_code[4:8]
            tr['specific_symbol'] = line[81:91].decode().lstrip('0')
            tr['client_name'] = line[97:117].decode(self.enc).rstrip()
            tr['maturity_date'] = self.parse_datetime(line[122:128].decode())

            if tr['other_acc_type'] in ('DEBIT'):  # TODO also STORNO_KREDIT
                tr['amount'] = -tr['amount']
            trans.append(tr)
        else:
            raise ValueError('Unknown type of account {}'.format(typ.decode()))

        if act:
            self.statement.start_balance = act['orig_amount']
            self.statement.start_date = act['orig_date']

            self.statement.end_balance = act['new_amount']
            self.statement.end_date = act['statement_date']
        else:
            return self.__generate_statement_line(tr)


class GPCPlugin(Plugin):
    def get_parser(self, fin):
        encoding = self.settings.get('charset', 'windows-1250')
        inf = open(fin, 'rb')
        return GPCParser(inf, encoding=encoding)



#    AlfaBank (https://www.alfabank.ru) plugin for ofxstatement
#
#    Copyright 2017 Alexander Gerasiov <gq@cs.msu.su>
#    Copyright 2017 Dmitry Pavlov <zeldigas@gmail.com>
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License version 3 as
#    published by the Free Software Foundation.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.

import csv
import re
from decimal import Decimal

from ofxstatement import statement
from ofxstatement.parser import StatementParser
from ofxstatement.plugin import Plugin

# Тип счёта;Номер счета;Валюта;Дата операции;Референс проводки;Описание операции;Приход;Расход;

# file format options
delimiter = ';'
default_encoding = 'cp1251'
fieldnames = ['acc_name', 'acc', 'currency', 'op_time', 'refnum', 'description', 'income', 'withdraw']
type_map = {
    u"Комиссия за ": 'SRVCHG'
}


def parse_type(type, amount):
    for filter in type_map.keys():
        if type.startswith(filter):
            return type_map[filter]

    result = None

    if amount > 0:
        result = 'DEBIT'
    elif amount < 0:
        result = 'CREDIT'

    return result


class AlfabankStatementParser(StatementParser):

    statement = None

    def __init__(self, fin):
        self.date_format = '%d.%m.%y'
        self.statement = statement.Statement()
        self.fin = fin
        # Skip 1st row with column's headers
        self.fin.readline()
        self.cur_record = 1
        self.user_date = False

    def split_records(self):
        return csv.DictReader(self.fin, delimiter=delimiter, fieldnames=fieldnames)

    def parse_record(self, line):
        transaction = statement.StatementLine()

        if not self.statement.account_id:
            self.statement.account_id = line['acc']

        if not self.statement.currency:
            self.statement.currency = line['currency']

        if not line['currency'] == self.statement.currency:
            print("Transaction %s currency '%s' differ from account currency '%s'." % (
            line['op_time'], line['currency'], self.statement.currency))
            return None

        transaction.date = self.parse_datetime(line['op_time'])
        date_user = self.try_find_user_date(line['description'])

        if self.user_date and date_user:
            transaction.date = self.parse_datetime(date_user)

        transaction.amount = self.get_amount(line['income'], line['withdraw'])

        transaction.trntype = parse_type(line['description'], transaction.amount)
        transaction.refnum = line['refnum']

        transaction.memo = line['description']

        if transaction.trntype:
            return transaction
        else:
            return None

    def get_amount(self, income, withdraw):
        income_val = Decimal(income.replace(',', '.'))
        withdraw_val = Decimal(withdraw.replace(',', '.'))
        if income_val == 0:
            return -withdraw_val
        else:
            return income_val

    @staticmethod
    def try_find_user_date(param):
        date_pattern = '\d{2}\.\d{2}\.\d{2}'
        m = re.search('{0} ({0})'.format(date_pattern), param)
        if m:
            return m.group(1)
        else:
            return None


class AlfabankPlugin(Plugin):
    """AlfaBank CSV (https://www.alfabank.ru)
    """

    def get_parser(self, fin):
        f = open(fin, 'r', encoding=self.settings.get('file_encoding', default_encoding))
        parser = AlfabankStatementParser(f)
        parser.statement.currency = self.settings.get('currency')
        parser.statement.account_id = self.settings.get('account')
        parser.statement.bank_id = self.settings.get('bank', 'Alfabank')
        parser.user_date = self.settings.get('user_date', 'true') == 'true'
        return parser



import os
import datetime

from ofxstatement.ui import UI
from ofxstatement.plugins.sberbank_csv import SberBankCSVPlugin


SAMPLES_DIR = os.path.join(os.path.dirname(__file__), 'samples')


def test_sberbank():
    plugin = SberBankCSVPlugin(UI(), {'currency': 'RUR'})
    s = plugin.get_parser(os.path.join(SAMPLES_DIR, 'sberbank.csv')).parse()

    assert s is not None

    assert s.account_id == u'Основная *6833'
    assert s.currency == 'RUR'
    assert s.bank_id == 'SberBank'

    assert len(s.lines) == 11

    assert all(l.amount for l in s.lines)

    assert s.lines[0].__dict__ == {
        'amount': -4320.4,
        'check_no': None,
        'date': datetime.datetime(2019, 10, 31, 0, 0),
        'date_user': datetime.datetime(2019, 10, 31, 0, 0),
        'id': None,
        'memo': 'SBOL перевод 4276****1234 И. ИВАН ИВАНОВИЧ, MOSCOW, RUS, 4829',
        'payee': None,
        'refnum': None,
        'trntype': 'CREDIT'
    }

    assert s.lines[3].__dict__ == {
        'amount': 9.01,
        'check_no': None,
        'date': datetime.datetime(2019, 6, 17, 0, 0),
        'date_user': datetime.datetime(2019, 6, 16, 0, 0),
        'id': None,
        'memo': 'SBERBANK ONL@IN VKLAD-KARTA , Moscow, RUS',
        'payee': None,
        'refnum': None,
        'trntype': 'DEBIT'
    }

    assert sum(l.amount for l in s.lines) == 7338.83



import doctest

from ofxstatement.plugins.danske import DanskeCsvStatementParser

def doctest_DanskeCsvStatementParser():
    """Test DanskeCsvStatementParser

    Open sample csv to parse
        >>> import os
        >>> csvfile = os.path.join(os.path.dirname(__file__),
        ...                        'samples', 'danske.csv')


    Create parser and parse
        >>> fin = open(csvfile, 'r', encoding='cp1257')
        >>> parser = DanskeCsvStatementParser(fin)
        >>> statement = parser.parse()

    Check what we've got:
        >>> len(statement.lines)
        4
        >>> statement.start_balance
        0.0
        >>> statement.end_balance
        0.0
        >>> statement.start_date
        datetime.datetime(2012, 3, 1, 0, 0)
        >>> statement.end_date
        datetime.datetime(2012, 3, 7, 0, 0)

    First line is a payment for incoming transaction:
        >>> l = statement.lines[0]
        >>> l.amount
        -7.8
        >>> l.memo
        'Paslaugų ir komisinių pajamos už gaunamus tarptautinius pervedimus USD'
        >>> l.date
        datetime.datetime(2012, 3, 1, 0, 0)
        >>> hash_expected = str(abs(hash((l.date,l.memo,l.amount))))
        >>> l.id == hash_expected
        True

    Second line is incoming money
        >>> l = statement.lines[1]
        >>> l.amount
        889.81
        >>> l.memo
        'ACME LLC'
        >>> l.date
        datetime.datetime(2012, 3, 1, 0, 0)

    Third line is a pament with amount less than 1 USD
        >>> l = statement.lines[2]
        >>> l.amount
        -0.46
        >>> l.memo
        'Mokestis už lėšų, gautų iš kitų LR registruotų bankų, ... 2012.03.06'

    Fourth is a transfer to other account
        >>> l = statement.lines[3]
        >>> l.memo
        'John Doe'
        >>> l.amount
        -881.55
    """

def test_suite(*args):
    return doctest.DocTestSuite(optionflags=(doctest.NORMALIZE_WHITESPACE|
                                             doctest.ELLIPSIS|
                                             doctest.REPORT_ONLY_FIRST_FAILURE|
                                             doctest.REPORT_NDIFF
                                             ))
load_tests = test_suite



#!/usr/bin/env python3.6

from openpyxl import load_workbook
from dataclasses import dataclass
from datetime import datetime
import logging

from ofxstatement.plugin import Plugin
from ofxstatement.parser import StatementParser
from ofxstatement.statement import (Statement, StatementLine,
                                    generate_transaction_id)

logging.basicConfig(level=logging.DEBUG)
logger = logging.getLogger('IntesaSP')


@dataclass
class Movimento:
    data_contabile: datetime
    data_valuta: datetime
    descrizione: str
    accrediti: float
    addebiti: float
    descrizione_estesa: str
    mezzo: str


class IntesaSanPaoloPlugin(Plugin):

    def get_parser(self, filename):
        parser = IntesaSanPaoloXlsxParser(filename)
        parser.statement.bank_id = self.settings.get('BIC', 'IntesaSP')
        return parser


class IntesaSanPaoloXlsxParser(StatementParser):

    def split_records(self):
        return self._get_movimenti()

    def __init__(self, filename):
        self.fin = filename
        self.statement = Statement()
        self.statement.account_id = self._get_account_id()
        self.statement.currency = self._get_currency()
        self.statement.start_balance = self._get_start_balance()
        self.statement.start_date = self._get_start_date()
        self.statement.end_balance = self._get_end_balance()
        self.statement.end_date = self._get_end_date()
        logging.debug(self.statement)

    def parse(self):
        return super(IntesaSanPaoloXlsxParser, self).parse()

    def parse_record(self, mov):
        stat_line = StatementLine(None,
                                  mov.data_contabile,
                                  mov.descrizione_estesa,
                                  mov.accrediti if mov.accrediti else
                                  mov.addebiti)
        stat_line.id = generate_transaction_id(stat_line)
        stat_line.date_user = mov.data_valuta
        stat_line.trntype = IntesaSanPaoloXlsxParser._get_transaction_type(mov)
        logging.debug(stat_line)
        return stat_line

    def _get_account_id(self):
        wb = load_workbook(self.fin)
        return wb['Lista Movimenti']['D8'].value

    def _get_currency(self):
        trans_map = {'Euro': 'EUR'}
        wb = load_workbook(self.fin)
        val = wb['Lista Movimenti']['D22'].value
        return trans_map[val]

    def _get_movimenti(self):
        wb = load_workbook(self.fin)
        starting_column = 'A'
        ending_column = 'G'
        starting_row = 30
        offset = 0

        while True:
            data = wb['Lista Movimenti'][
                        '{}{}:{}{}'.format(starting_column,
                                           starting_row + offset,
                                           ending_column,
                                           starting_row + offset
                                           )]

            values = [*map(lambda x: x.value, data[0])]
            logging.debug(values)
            if not values[0]:
                break
            else:
                yield Movimento(*values)
                offset += 1

    def _get_transaction_type(movimento):
        trans_map = {'Pagamento pos': 'POS',
                     'Pagamento effettuato su pos estero': 'POS',
                     'Accredito beu con contabile': 'XFER',
                     'Canone mensile base e servizi aggiuntivi': 'SRVCHG',
                     'Prelievo carta debito su banche del gruppo': 'CASH',
                     'Prelievo carta debito su banche italia/sepa': 'CASH',
                     'Versamento contanti su sportello automatico':
                     'DIRECTDEP',
                     'Comm.prelievo carta debito italia/sepa': 'SRVCHG',
                     'Commiss. su beu internet banking': 'SRVCHG',
                     'Pagamento telefono': 'PAYMENT',
                     'Pagamento mav via internet banking': 'PAYMENT',
                     'Pagamento bolletta cbill': 'PAYMENT',
                     'Beu tramite internet banking': 'PAYMENT',
                     'Commissione bolletta cbill': 'SRVCHG',
                     'Storno pagamento pos': 'POS',
                     'Storno pagamento pos estero': 'POS',
                     'Versamento contanti su sportello automatico': 'ATM'}
        return trans_map[movimento.descrizione]

    def _get_start_balance(self):
        wb = load_workbook(self.fin)
        return float(wb['Lista Movimenti']['E11'].value)

    def _get_end_balance(self):
        wb = load_workbook(self.fin)
        return float(wb['Lista Movimenti']['E12'].value)

    def _get_start_date(self):
        wb = load_workbook(self.fin)
        date = wb['Lista Movimenti']['D11'].value
        return datetime.strptime(date, '%d.%m.%Y')

    def _get_end_date(self):
        wb = load_workbook(self.fin)
        date = wb['Lista Movimenti']['D12'].value
        return datetime.strptime(date, '%d.%m.%Y')



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

from distutils.core import setup

version = '1.0.0'
with open('README.rst') as f:
    long_description = f.read()

setup(name='ofxstatement-paypal',
      version=version,
      author='Alexander Krasnukhin',
      author_email='the.malkolm@gmail.com',
      url='https://github.com/themalkolm/ofxstatement-paypal',
      description=('ofxstatement plugins for paypal'),
      long_description=long_description,
      license='Apache License 2.0',
      keywords=['ofx', 'ofxstatement', 'paypal'],
      classifiers=[
          'Development Status :: 3 - Alpha',
          'Programming Language :: Python :: 3',
          'Natural Language :: English',
          'Topic :: Office/Business :: Financial :: Accounting',
          'Topic :: Utilities',
          'Environment :: Console',
          'Operating System :: OS Independent'
      ],
      packages=['ofxstatement', 'ofxstatement.plugins'],
      namespace_packages=['ofxstatement', 'ofxstatement.plugins'],
      entry_points={
          'ofxstatement': ['paypal = ofxstatement.plugins.paypal:PayPalPlugin']
      },
      install_requires=['ofxstatement'],
      test_suite='ofxstatement.plugins.tests',
      include_package_data=True,
      zip_safe=True
      )



"""Parser for LITAS-ESIS csv statement"""

import csv

from ofxstatement.parser import CsvStatementParser
from ofxstatement.plugin import Plugin

LINETIME_HEADER = "000"
LINETYPE_TRANSACTION = "010"
LINETYPE_SUMMARY = "020"

SUMMARY_START = "LikutisPR"
SUMMARY_END = "LikutisPB"


class LitasEsisCsvDialect(object):
    delimiter = "\t"
    quotechar = None
    escapechar = None
    doublequote = False
    skipinitialspace = False
    lineterminator = '\r\n'
    quoting = csv.QUOTE_NONE


class LitasEsisCsvStatementParser(CsvStatementParser):
    date_format = "%Y%m%d"
    mappings = {"date": 2,
                "amount": 4,
                "id": 10,
                "payee": 17,
                "memo": 13,
                }
    charset = 'cp1257'

    def parse_float(self, value):
        return int(value) / 100.0

    def split_records(self):
        return csv.reader(self.fin, dialect=LitasEsisCsvDialect)

    def parse_record(self, line):
        # print(line)

        linetype = line[0]
        stmt = self.statement

        if linetype == LINETIME_HEADER:
            # Get basic account information
            stmt.currency = line[17]
            stmt.bank_id = line[3]
            stmt.account_id = line[16] + line[17]
            return None

        elif linetype == LINETYPE_TRANSACTION:
            # parse transaction line in standard fasion
            stmtline = super(LitasEsisCsvStatementParser, self).parse_record(line)
            if line[6] == "D":
                stmtline.amount = -stmtline.amount
            return stmtline

        elif linetype == LINETYPE_SUMMARY:
            summarytype = line[1]
            if summarytype == SUMMARY_START:
                stmt.start_balance = self.parse_float(line[4])
                stmt.start_date = self.parse_datetime(line[2])
            elif summarytype == SUMMARY_END:
                stmt.end_balance = self.parse_float(line[4])
                stmt.end_date = self.parse_datetime(line[2])
            return None

    def swap_payee_and_memo(self):
        payee, memo = self.mappings['payee'], self.mappings['memo']
        self.mappings['payee'] = memo
        self.mappings['memo'] = payee


class LitasEsisPlugin(Plugin):
    """Standard Lithuanian LITAS-ESIS format"""

    def get_parser(self, fin):
        encoding = self.settings.get('charset', 'utf-8')
        f = open(fin, 'r', encoding=encoding)
        parser = LitasEsisCsvStatementParser(f)
        if 'swap-payee-and-memo' in self.settings:
            parser.swap_payee_and_memo()
        return parser



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
test_ofxstatement-simple
----------------------------------

Tests for `ofxstatement-simple` module.
"""
from datetime import date
import logging

import unittest
import uuid
import os
from ofxstatement.plugins.simple import SimpleBankPlugin, is_zero

logging.basicConfig(level=logging.DEBUG)
logger = logging.getLogger('test_simple')

DATA_DIR = os.path.join(os.path.dirname(__file__), 'test_data')
BAL_0_BAL_JSON_PATH = os.path.join(DATA_DIR, 'exported_transactions.json')
BAL_500_BAL_JSON_PATH = os.path.join(DATA_DIR, 'exported_transactions-500.json')
BAL_0_AMT_JSON_PATH = os.path.join(DATA_DIR, 'exported_transactions-0-amount.json')


# Shared Methods #


class TestSimpleBankStatement(unittest.TestCase):
    def test0Balance(self):
        """Statement starts with a zero balance."""
        stmt = SimpleBankPlugin(None, {}).get_parser(BAL_0_BAL_JSON_PATH).parse()

        self.assertAlmostEqual(0.0, stmt.start_balance)
        self.assertAlmostEqual(20.03, stmt.end_balance)
        self.assertEqual(date(2013, 2, 6), to_date(stmt.start_date))
        self.assertEqual(date(2014, 8, 30), to_date(stmt.end_date))
        self.assertEqual(19, len(stmt.lines))
        for stline in stmt.lines:
            self.assertIsNotNone(stline.memo)
            self.assertFalse(is_zero(stline.amount))
            self.assertTrue(date(2014, 8, 30) >= to_date(stline.date) >= date(2013, 2, 6))

    def test500Balance(self):
        """Statement starts with a $500 balance."""
        stmt = SimpleBankPlugin(None, {}).get_parser(BAL_500_BAL_JSON_PATH).parse()

        self.assertAlmostEqual(500.0, stmt.start_balance)
        self.assertAlmostEqual(20.03, stmt.end_balance)
        self.assertEqual(date(2013, 3, 3), to_date(stmt.start_date))
        self.assertEqual(date(2014, 8, 30), to_date(stmt.end_date))
        self.assertEqual(15, len(stmt.lines))
        for stline in stmt.lines:
            self.assertIsNotNone(stline.memo)
            self.assertFalse(is_zero(stline.amount))
            self.assertTrue(date(2014, 8, 30) >= to_date(stline.date) >= date(2013, 3, 3))

    def test0AmountExclude(self):
        """Statement starts with a zero-amount balance that's filtered out."""
        stmt = SimpleBankPlugin(None, {'account': '12345', 'filter_zeros': 'true'}
                                ).get_parser(BAL_0_AMT_JSON_PATH).parse()

        self.assertEqual(stmt.account_id, '12345')
        self.assertEqual(stmt.bank_id, 'Simple')
        self.assertAlmostEqual(480.0, stmt.start_balance)
        self.assertAlmostEqual(20.03, stmt.end_balance)
        self.assertEqual(date(2013, 3, 30), to_date(stmt.start_date))
        self.assertEqual(date(2014, 8, 30), to_date(stmt.end_date))
        self.assertEqual(14, len(stmt.lines))
        for stline in stmt.lines:
            self.assertIsNotNone(stline.memo)
            self.assertFalse(is_zero(stline.amount))
            self.assertTrue(date(2014, 8, 30) >= to_date(stline.date) >= date(2013, 3, 30))

    def test0AmountInclude(self):
        """Statement starts with a zero-amount balance that's included."""
        stmt = SimpleBankPlugin(None, {'account': '12345', 'filter_zeros': 'false'}
                                ).get_parser(BAL_0_AMT_JSON_PATH).parse()

        self.assertEqual(stmt.account_id, '12345')
        self.assertEqual(stmt.bank_id, 'Simple')
        self.assertAlmostEqual(480.0, stmt.start_balance)
        self.assertAlmostEqual(20.03, stmt.end_balance)
        self.assertEqual(date(2013, 3, 3), to_date(stmt.start_date))
        self.assertEqual(date(2014, 8, 30), to_date(stmt.end_date))
        self.assertEqual(15, len(stmt.lines))
        for stline in stmt.lines:
            self.assertIsNotNone(stline.memo)
            if is_zero(stline.amount):
                self.assertEqual(date(2013, 3, 3), to_date(stline.date))
            self.assertTrue(date(2014, 8, 30) >= to_date(stline.date) >= date(2013, 3, 3))

def to_date(dt):
    """Creates a comparable date object from the given datetime."""
    return date(dt.year, dt.month, dt.day)



from ofxstatement.parser import CsvStatementParser
from ofxstatement.plugin import Plugin
from ofxstatement import statement
import csv

class DKBCCCsvStatementParser(CsvStatementParser):
    mappings = {"date": 1, "memo": 3, "amount": 4}
    date_format = "%d.%m.%Y"

    def split_records(self):
        return csv.reader(self.fin, delimiter=';')

    def parse_record(self, line):
        if self.cur_record  == 3:
            self.statement.start_date=self.parse_datetime(line[1])
            return None
        if self.cur_record == 5:
            self.statement.end_balance=self.parse_float(line[1].replace(' EUR',''))
            return None
        if self.cur_record == 6:
            self.statement.end_date=self.parse_datetime(line[1])
            return None

        if self.cur_record == 8:
            self.statement.currency=line[4].strip('Betrag (').strip(')')
            return None

        if self.cur_record <= 8:
            return None
        line[4]=line[4].replace('.','').replace(',','.')        
        # fill statement line according to mappings
        sl = super(DKBCCCsvStatementParser, self).parse_record(line)
        return sl

class DKBCCPlugin(Plugin):
    """
    Deutsche Kreditbank AG, CSV of creditcard statements
    """
    def get_parser(self, fin):
        f = open(fin, "r",encoding='iso-8859-1')
        parser=DKBCCCsvStatementParser(f)
        parser.statement.account_id = self.settings['account']
        parser.statement.bank_id = self.settings.get('bank', 'DKB_VISA')
        return parser




import doctest

from ofxstatement.plugins.flatex import FlatexCsvStatementParser

def doctest_FlatexCsvStatementParser():
    """Test FlatexCsvStatementParser

    Open sample csv to parse
        >>> import os
        >>> csvfile = os.path.join(os.path.dirname(__file__),
        ...                        'samples', 'flatex.csv')

    Create parser object and parse:
        >>> fin = open(csvfile, 'r', encoding='iso-8859-1')
        >>> parser = FlatexCsvStatementParser(fin)
        >>> statement = parser.parse()

    Check what we've got:
        >>> statement.account_id
        >>> len(statement.lines)
        12
        >>> statement.start_balance
        >>> statement.start_date
        >>> statement.end_balance
        >>> statement.end_date
        >>> statement.currency
        'EUR'

    Check first line
        >>> l = statement.lines[0]
        >>> l.amount
        -1672.5
        >>> l.payee 
        >>> l.memo
        'AusfÃ¼hrung ORDER Kauf,CNE1000004Y2 12345678,TA-Nr.: 12345678'
        >>> l.date
        datetime.datetime(2015, 4, 7, 0, 0)

    Check one more line:
        >>> l=statement.lines[2]
        >>> l.amount
        -13.49
        >>> l.payee
        >>> l.memo
        'GebÃ¼hren Ordernummer,12345678 Transnr 12345678,TA-Nr.: 12345678'
        >>> l.date
        datetime.datetime(2015, 4, 8, 0, 0)

    Check one more line with slashes in memo:
        >>> l=statement.lines[8]
        >>> l.amount
        8.55
        >>> l.memo
        'ErtrÃ¤gnisausschÃ¼ttung,LU0136412771,TA-Nr.: 12345678'
        >>> l.date
        datetime.datetime(2015, 4, 9, 0, 0)
              
    """

def test_suite(*args):
    return doctest.DocTestSuite(optionflags=(doctest.NORMALIZE_WHITESPACE|
                                             doctest.ELLIPSIS|
                                             doctest.REPORT_ONLY_FIRST_FAILURE|
                                             doctest.REPORT_NDIFF
                                             ))
load_tests = test_suite

if __name__ == "__main__":
    doctest.testmod()




# -*- coding: utf-8 -*-

import re
import itertools
import logging

from datetime import datetime
from openpyxl import load_workbook
from io import BytesIO

from ofxstatement.parser import StatementParser
from ofxstatement.plugin import Plugin
from ofxstatement.statement import Statement, StatementLine, BankAccount

class ArgentaStatementParser(StatementParser):
    header = ['Rekening', 'Boekdatum', 'Valutadatum', 'Referentie',
    'Beschrijving', 'Bedrag', 'Munt', 'Verrichtingsdatum',
    'Rekening tegenpartij', 'Naam tegenpartij', 'Mededeling']
    dict_transaction_types = {
        'Inkomende overschrijving': 'CREDIT',
        'Uitgaande overschrijving': 'DEBIT',
        'Uitgaande instantoverschrijving': 'DEBIT',
        'Debet ten voordele van BCC': 'DEBIT',
        'Betaling Bancontact': 'POS',
        'Betaling Maestro': 'POS',
        'Bestendige opdracht': 'REPEATPMT',
        'SEPA-domiciliëring': 'REPEATPMT',
        'Doorlopende betalingsopdracht': 'REPEATPMT',
        'Storting kaarttransactie': 'XFER',
        'Opname Bancontact': 'ATM',
        'Opname Maestro': 'ATM',
        'Interestberekening': 'INT'
    }# TODO Add more transaction types as I encounter them.
    # Transaction types used in <TRNTYPE>: xml.coverpages.org/OFEXFIN2.html#_Ref377532222

    def __init__(self, fin):
        """Create a new ArgentaStatementParser instance.
        """
        
        self.col_index = dict(zip(self.header, range(0, 11)))

        with open(fin, "rb") as f:
            in_mem_file = BytesIO(f.read())
        
        self.workbook = load_workbook(filename=in_mem_file, read_only=True)
        self.sheet = self.workbook.active
        # dimensions are incorrectly set in the XLSX-file
        self.sheet.reset_dimensions()
        
        self.validate()

        self.statement = self.parse_statement()
    
    def __enter__(self):
        return self
    
    def __exit__(self, exc_type, exc_value, traceback):
        self.workbook.close()

    def validate(self):
        """
        Naive validation to make sure that the XLSX document is structured
        the way it was when this parser was written.

        :raises ValueError if workbook has invalid format
        """

        try:
            self._validate()
        except AssertionError as e:
            raise ValueError(e)

    def _validate(self):
        """
        Headers of the XLSX file:
        [Rekening, Boekdatum, Valutadatum, Referentie,
         Beschrijving, Bedrag, Munt, Verrichtingsdatum,
         Rekening tegenpartij, Naam tegenpartij, Mededeling]
        Row of sample data:
        [BE46 5390 0754 7034, 24-04-2018, 24-04-2018, 7SLLCW65XDZQU891,
         Uitgaande overschrijving, -125,00, EUR, 24-04-2018,
         BE68 5390 0754 7034, Adams White, Comment of transaction]
        """
        
        logging.debug('Verifying that the sheet has at least 2 rows.')
        top_two_rows = list(itertools.islice(self.sheet.iter_rows(), 2))
        assert len(top_two_rows) == 2

        logging.debug('Verifying that the first row has 11 cells.')
        assert len(top_two_rows[1]) == 11

        logging.debug('Verifying statements header.')
        statement_header_row = [c.value for c in top_two_rows[0]]
        assert self.header == statement_header_row

        logging.debug('Verifying account numbers are IBAN Belgian formatted (A2 and I2).')
        first_stmt_row = [c.value for c in top_two_rows[1]]
        assert BankAccountIban.is_valid(first_stmt_row[self.col_index['Rekening']])
        if first_stmt_row[self.col_index['Rekening tegenpartij']]:
            assert BankAccountIban.is_valid(first_stmt_row[self.col_index['Rekening tegenpartij']])
        
        logging.debug('Verifying statement date is a date (H2).')
        assert isinstance(first_stmt_row[self.col_index['Verrichtingsdatum']], datetime)
        
        logging.debug('Verifying account id is equal for every transaction (column A).')
        BankAccountIbans_are_equal = True
        currencies_are_equal = True
        for row in self.sheet.iter_rows(min_row=3):
            # Rekening A
            if row[0].value != first_stmt_row[self.col_index['Rekening']]:
                BankAccountIbans_are_equal = False
                break
            # Munt G
            if row[6].value != first_stmt_row[self.col_index['Munt']]:
                currencies_are_equal = False
                break
        
        assert BankAccountIbans_are_equal
        logging.debug('Verifying currency is equal for every transaction (column G).')
        assert currencies_are_equal

        logging.debug('Everything is OK!')

    def parse_statement(self):
        statement = Statement()
        
        account_holder = BankAccountIban(self.sheet['A2'].value)
        statement.bank_id = account_holder.bank_id
        statement.account_id = account_holder.acct_id
        statement.currency = self.sheet['G2'].value
        
        return statement

    def split_records(self):
        # Skip the header
        for row in self.sheet.iter_rows(min_row=2):
            yield [c.value for c in row]

    def parse_record(self, row):
        stmt_line = StatementLine()
        
        stmt_line.id = row[self.col_index['Referentie']]
        stmt_line.date = row[self.col_index['Verrichtingsdatum']]
        stmt_line.memo = row[self.col_index['Mededeling']]
        stmt_line.amount = row[self.col_index['Bedrag']]
        stmt_line.payee = row[self.col_index['Naam tegenpartij']]
        stmt_line.refnum = row[self.col_index['Referentie']]
        
        stmt_line.trntype = self.dict_transaction_types.get(row[self.col_index['Beschrijving']], 'OTHER')
        if stmt_line.trntype == 'OTHER':
            logging.warning('Other transaction type found: '+row[self.col_index['Beschrijving']])
        
        try:
            stmt_line.bank_account_to = BankAccountIban(row[self.col_index['Rekening tegenpartij']])
        except ValueError:
            pass
        
        return stmt_line

class BankAccountIban(BankAccount):
    """Represents a account id formatted in Belgian IBAN.
    Eg: BE46539007547034
    """    
    def __init__(self, acct_id):
        if acct_id and BankAccountIban.is_valid(acct_id):
            acct_id_canon = acct_id.replace(" ", "")
            super().__init__(bank_id=acct_id_canon[4:7], acct_id=acct_id_canon)
            self.acct_key = acct_id_canon[2:4]
        else:
            raise ValueError("acct_id is not a valid Belgian IBAN string.")
    
    iban_pattern = re.compile('^[A-Z]{2}\d{2} ?\d{4} ?\d{4} ?\d{4} ?[\d]{0,2}$')
    
    @staticmethod
    def is_valid(acct_id):
        """Returns True if the given account id is formatted as 
        for example 'BE46 5390 0754 7034' or 'BE46539007547034'
        """
        return BankAccountIban.iban_pattern.match(acct_id)

class ArgentaPlugin(Plugin):
    """Parses XLSX file from Belgian bank Argenta
    """
    def get_parser(self, fin):
        return ArgentaStatementParser(fin)



import doctest

from ofxstatement.plugins.vtb import VtbStatementParser


def doctest_VtbParser():
    """Test VtbStatementParser

    Lets define VTB sample csv to parse and write it to file-like object

        >>> import io
        >>> csv = '''Начало периода;2019-07-07
        ... Конец периода;2019-07-13
        ...
        ... Выписка по счету/карте;Номер
        ... Карта с кредитной составляющей;'462235******0069
        ...
        ...
        ... Валюта;Баланс на конец периода;Поступления;Списания;Заблокировано
        ... RUR;82 604,01;0,00;-17 351,00;-2 426,96
        ...
        ...
        ... Номер карты/счета/договора;Дата операции;Дата обработки;Сумма операции;Валюта операции;Сумма пересчитанная в валюту счета;Валюта счета;Основание;Статус
        ... '462235******7428;2019-07-06 00:00:00;2019-07-08;-4,50;EUR;-336,15;RUR;Карта *1234 payee1;Исполнено
        ... '462235******7428;2019-07-06 00:00:00;2019-07-07;-3,80;EUR;-283,48;RUR;Карта *1234 payee2;Исполнено
        ... '462235******7428;2019-07-05 00:00:00;2019-07-07;-300,00;RUR;-300,00;RUR;Карта *1234 PAY MTS RU TOPUP 1234;Исполнено
        ... '462235******7428;2019-07-13 10:48:55;;-600,00;RUR;-600,00;RUR;Карта *1234 MEGAFON TOPUP  1234;В обработке
        ... '''
        >>> f = io.StringIO(csv)

    Create and configure csv parser:
        >>> parser = VtbStatementParser(f)
        >>> parser.statement.bank_id = 'VTB'

    And parse csv:
        >>> statement = parser.parse()
        >>> statement.account_id
        '462235******0069'
        >>> statement.start_date
        datetime.datetime(2019, 7, 7, 0, 0)
        >>> statement.end_date
        datetime.datetime(2019, 7, 13, 0, 0)
        >>> statement.start_balance
        Decimal('99955.01')
        >>> statement.end_balance
        Decimal('82604.01')
        >>> len(statement.lines)
        4
        >>> statement.lines[0].amount
        Decimal('-336.15')
        >>> statement.lines[0].memo
        'Карта *1234 payee1'
        >>> statement.lines[0].date
        datetime.datetime(2019, 7, 8, 0, 0)
        >>> statement.lines[0].payee
        'payee1'

        >>> statement.lines[1].amount
        Decimal('-283.48')
        >>> statement.lines[1].date
        datetime.datetime(2019, 7, 7, 0, 0)
        >>> statement.lines[1].memo
        'Карта *1234 payee2'
        >>> statement.lines[1].payee
        'payee2'

        >>> statement.lines[2].amount
        Decimal('-300.00')
        >>> statement.lines[2].memo
        'Карта *1234 PAY MTS RU TOPUP 1234'
        >>> statement.lines[2].date
        datetime.datetime(2019, 7, 7, 0, 0)
        >>> statement.lines[2].payee
        'PAY MTS RU TOPUP 1234'

        >>> statement.lines[3].amount
        Decimal('-600.00')
        >>> statement.lines[3].memo
        'Карта *1234 MEGAFON TOPUP  1234'
        >>> statement.lines[3].date
        >>> statement.lines[3].payee
        'MEGAFON TOPUP  1234'

    """


def doctest_VtbParser_userdates():
    """Test generic VtbStatementParser

    Lets define VTB sample csv to parse and write it to file-like object

        >>> import io
        >>> csv = '''Начало периода;2019-07-07
        ... Конец периода;2019-07-13
        ...
        ... Выписка по счету/карте;Номер
        ... Карта с кредитной составляющей;'462235******0069
        ...
        ...
        ... Валюта;Баланс на конец периода;Поступления;Списания;Заблокировано
        ... RUR;82 604,01;0,00;-17 351,00;-2 426,96
        ...
        ...
        ... Номер карты/счета/договора;Дата операции;Дата обработки;Сумма операции;Валюта операции;Сумма пересчитанная в валюту счета;Валюта счета;Основание;Статус
        ... '462235******7428;2019-07-06 00:00:00;2019-07-08;-4,50;EUR;-336,15;RUR;Карта *1234 payee1;Исполнено
        ... '''
        >>> f = io.StringIO(csv)

    Create and configure csv parser:
        >>> parser = VtbStatementParser(f)
        >>> parser.statement.bank_id = 'VTB'
        >>> parser.user_date = True

    And parse csv:
        >>> statement = parser.parse()
        >>> len(statement.lines)
        1
        >>> statement.lines[0].amount
        Decimal('-336.15')
        >>> statement.lines[0].memo
        'Карта *1234 payee1'
        >>> statement.lines[0].date
        datetime.datetime(2019, 7, 6, 0, 0)
    """

def test_suite(*args):
    return doctest.DocTestSuite(optionflags=(doctest.NORMALIZE_WHITESPACE|
                                             doctest.ELLIPSIS|
                                             doctest.REPORT_ONLY_FIRST_FAILURE|
                                             doctest.REPORT_NDIFF
                                             ))


import csv
import re

from ofxstatement import statement
from ofxstatement.parser import CsvStatementParser
from ofxstatement.plugin import Plugin
from ofxstatement.statement import Statement, StatementLine, recalculate_balance


class DBPLPlugin(Plugin):
    """Deutsche Bank Polska (www.deutschebank.pl) bank CSV history plugin 
    """

    def get_parser(self, filename):
        encoding = self.settings.get('charset', 'cp1250')
        f = open(filename, "r", encoding=encoding)
        parser = DBPLParser(f)
        parser.statement.currency = self.settings.get('currency', 'PLN')
        parser.statement.account_id = self.settings.get('account', '')
        parser.statement.bank_id = self.settings.get('bank', 'DEUTPLPK')
        return parser

class DBPLParser(CsvStatementParser):
    mappings = {"date": 0,
                "amount": 3}
    
    date_format = "%Y-%m-%d"
    regex = re.compile('^(.*);.* (Adresat|Nadawca): (.*) Treść:(.*)')
    
    def parse(self):
        stmt = super(DBPLParser, self).parse()
        recalculate_balance(stmt)
        return stmt
        
    def split_records(self):
        return csv.reader(self.fin, delimiter=';', quotechar='"')
    
    def parse_record(self, line):
        
        if not self.statement.currency:
            self.statement.currency = line[4]
        
        sl = super(DBPLParser, self).parse_record(line)
        
        opis = line[2]
        for n in (242, 179, 161, 132, 80):
            if n < len(opis) and opis[n] == ' ':
                #print(str(n) + ": " + opis[:n] + "~" + opis[n+1:])
                opis = opis[:n] + opis[n+1:]
        
        opis = re.sub("\s+", " ", opis).strip()
        m = self.regex.match(opis)
        if m:
            sl.payee = ' - '.join([m.group(1), m.group(4)])
            sl.memo = m.group(3)
        else:
            sl.payee = opis
        
        sl.id = statement.generate_transaction_id(sl)
        
        if opis.startswith("PRZELEW"):
            sl.trntype = "XFER"
        elif opis.startswith("OPERACJA KARTĄ Treść: Wypłata"):
            sl.trntype = "ATM"
        elif opis.startswith("OPERACJA KARTĄ Treść: Zakup"):
            sl.trntype = "DEBIT"
        elif opis.startswith("Podatek"):
            sl.trntype = "FEE"
        elif opis.startswith("OPŁAT"):
            sl.trntype = "SRVCHG"
        elif opis.startswith("Kapitalizacja"):
            sl.trntype = "INT"
   
        return sl
    
    def parse_float(self, value):
        return super(DBPLParser, self).parse_float(re.sub("[ .a-zA-Z]", "", value).replace(",", "."))